function do_all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% analyze non normal dynamics of the THC in the gfdl coupled model.
% Eli tziperman, June 2006

% (1) read yearly averaged netcdf files from the gfdl coupled model
%     (CM2)

% (2) calculate EOFs for the T,S variability

% (3) fit a linear model to the evolution of the first N_EOF EOF's
%     principle components

% (4) analyze the linearized (hopefully non normal) dynamics, find
% optimal initial conditions.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all;
addpath('/home/eli/matlab/seawater');
addpath('/home/eli/matlab/m_map');

% profile -detail 'builtin' on

write_to_file=1;
run_num=input(['which case? (1-13 for cases presented in paper, 0 for defaults): ']);
if not(exist(sprintf('Output/run%2.2d/Figures',run_num),'dir'));
  mkdir(sprintf('Output/run%2.2d',run_num));
  mkdir(sprintf('Output/run%2.2d/Figures',run_num));
end
if write_to_file==1;
  % create output directory for this run if needed:
  out_file=fopen(sprintf('Output/run%2.2d/output.txt',run_num),'w');
else
  out_file=1;
end
fprintf(out_file,'analyzing run %d\n',run_num);


% set parameters:
[do_read_raw_netcdf_data, do_read_TS_data, ncenturies ...
              ,do_avg_instead_of_subsample ...
              ,read_mask, print_raw_mask, do_contour_TS, do_calc_EOFs ...
              ,do_apply_grid_weights, do_normalize_by_std ...
              ,do_calc_PCs, do_calc_PCs_from_smoothed_data, do_smooth_PCs ...
              ,do_dimensionalize_EOFs, do_use_normalized_PCs ...
              ,do_analyze_THC_index, do_filter_in_analyze_THC_index, do_plot_PCs ...
              ,which_THC_to_use, do_calc_linear_fit, plot_in_fit_linear_model ...
              ,plot_obs_and_predicted_PCs, n_PCs_to_fit ...
              ,smooth_b4_plotting_predicted_PCs, prediction_start_interval ...
              ,prediction_lead_time_for_fit, tau_max ...
              ,do_eval_fit_quality, do_tau_test, do_analyze_linarized_dynamics ...
              ,do_plot_optimal_ic, do_plot_linear_model_oscillation ...
              ,do_contour_EOFs, data_dir, netcdf_path ...
              ,subsample_x, subsample_y, subsample_z, subsample_t, t_run_avg ...
              ,do_smooth_data_in_time,nsmooth_passes ...
              ,nx_b, nx_e, ny_b, ny_e, nz_b, nz_e, nt_b, nt_e, n_EOFs]= ...
              set_parameters(out_file,run_num);

% read data:
if do_read_TS_data
  [T,S,x,y,z,mask,mask3d,nt,nx,ny,nz]= ...
      read_and_subsample_data(do_read_raw_netcdf_data,do_read_TS_data ...
              ,ncenturies, do_contour_EOFs, read_mask, print_raw_mask ...
              ,do_contour_TS, data_dir,netcdf_path,do_smooth_data_in_time ...
              ,do_avg_instead_of_subsample ...
              ,subsample_x,subsample_y,subsample_z,subsample_t,nsmooth_passes ...
              ,nx_b,nx_e,ny_b,ny_e,nz_b,nz_e,nt_b,nt_e,t_run_avg,out_file,run_num);
end

if do_calc_EOFs

  load(sprintf('Output/run%2.2d/masks.mat',run_num));

  % calculate and save T,S as a vector for both smoothed and nonsmoothed
  % data using pre-processed data saved by read_and_subsample_data:
  fprintf(out_file,'reading pre-processed nonsmoothed Atlantic-only data...');
  load(sprintf('Output/run%2.2d/atlantic_data_nonsmoothed.mat',run_num));
  fprintf(out_file,' done.\n');
  use_smoothed_TS_data=0;
  [im,jm,km,nsea]=rearrange_TS_into_vector_X ...
      (T,S,x,y,z,mask,nt,nx,ny,nz,use_smoothed_TS_data ...
     ,do_apply_grid_weights,do_normalize_by_std,out_file,run_num);
  save(sprintf('Output/run%2.2d/im_jm_km.mat',run_num),'im','jm','km','nsea');
  % repeat, with smoothed data:
  fprintf(out_file,'reading pre-processed smoothed Atlantic-only data...');
  load(sprintf('Output/run%2.2d/atlantic_data_smoothed.mat',run_num));
  fprintf(out_file,' done.\n');
  use_smoothed_TS_data=1;
  [im,jm,km,nsea]=rearrange_TS_into_vector_X ...
      (T,S,x,y,z,mask,nt,nx,ny,nz,use_smoothed_TS_data ...
     ,do_apply_grid_weights,do_normalize_by_std,out_file,run_num);
  
  % calculate autocorrelation matrix:
  load(sprintf('Output/run%2.2d/X_smoothed_data.mat',run_num));
  C=calc_autocorrelation(X,nt,out_file);

  % clear variables to free some memory:
  clear T S

  % use eigs calculate EOFs:
  [EOFs_T,EOFs_S]=calculate_EOFs(n_EOFs,C,im,jm,km,nsea ...
		     ,x,y,z,nx,ny,nz,out_file,run_num);
end

% Contour subsampled T,S and their anomalies from long term mean.
% this has to happen after calculating EOFs, because T_avg and
% S_avg are calculated there...
if do_contour_TS
  contour_TS(out_file,run_num);
end
  
if do_calc_PCs
  % calculate time series of EOF's principle components
  PC=calc_principal_components ...
     (do_plot_PCs,do_calc_PCs_from_smoothed_data,out_file,run_num);
end

if do_dimensionalize_EOFs
  dimensionalize_EOFs(do_normalize_by_std,do_apply_grid_weights,out_file,run_num);
end

if do_contour_EOFs
  % plot EOFs:
  load(sprintf('Output/run%2.2d/masks.mat',run_num));
  load(sprintf('Output/run%2.2d/EOFs_dimensional.mat',run_num));
  load(sprintf('Output/run%2.2d/T_avg_and_std.mat',run_num));
  load(sprintf('Output/run%2.2d/W_dimensionalize.mat',run_num));
  plot_eofs_what_to_contour='eofs';
  plot_3dTS(EOFs_T,EOFs_S,x,y,z,mask3d,n_EOFs,nx,ny,nz ...
            ,plot_eofs_what_to_contour,out_file,run_num);
  plot_eofs_what_to_contour='weights';
  nplots=1;
  plot_3dTS(W_T_dimensionalize,W_S_dimensionalize,x,y,z,mask3d,nplots,nx,ny,nz ...
            ,plot_eofs_what_to_contour,out_file,run_num);
end

if do_analyze_THC_index
  analyze_THC_index(out_file,do_filter_in_analyze_THC_index ...
		    ,do_use_normalized_PCs,which_THC_to_use ...
                    ,n_PCs_to_fit,t_run_avg,do_smooth_PCs,run_num)
end

% fit a linear model to the evolution of the PC state vector:
if do_calc_linear_fit
  N_seg=1; t_fit_start=[1]; t_fit_end=[1500];
  fit_AR1=0;
  A=fit_linear_model(n_PCs_to_fit,prediction_lead_time_for_fit ...
     ,do_use_normalized_PCs,do_smooth_PCs ...
     ,N_seg,t_fit_start,t_fit_end,plot_in_fit_linear_model,fit_AR1,out_file,run_num);
  fit_AR1=1;
  AR1=fit_linear_model(n_PCs_to_fit,prediction_lead_time_for_fit ...
     ,do_use_normalized_PCs,do_smooth_PCs ...
     ,N_seg,t_fit_start,t_fit_end,plot_in_fit_linear_model,fit_AR1,out_file,run_num);
  save(sprintf('Output/run%2.2d/linearized_model_A_AR1.mat',run_num),'A','AR1');
end

% how good is the fit?
if do_eval_fit_quality
  load(sprintf('Output/run%2.2d/linearized_model_A_AR1.mat',run_num));
  evaluate_fit_quality(A,AR1,n_PCs_to_fit,prediction_lead_time_for_fit ...
    ,plot_in_fit_linear_model,tau_max,do_use_normalized_PCs,do_smooth_PCs ...
    ,plot_obs_and_predicted_PCs,smooth_b4_plotting_predicted_PCs ...
    ,which_THC_to_use ...
    ,prediction_start_interval,do_tau_test,out_file,run_num);
end

% analyze the linear matrix:
if do_analyze_linarized_dynamics
  use_THC_norm_kernel=0;
  analyze_linearized_nonormal_dynamics ...
      (use_THC_norm_kernel,tau_max,prediction_lead_time_for_fit,run_num,out_file);
  plot_IC_and_max_anomaly_in_PC_space(use_THC_norm_kernel,out_file,run_num)

  plot_A_eigenvectors_3dTS(run_num,out_file);
  
  use_THC_norm_kernel=1;
  analyze_linearized_nonormal_dynamics ...
      (use_THC_norm_kernel,tau_max,prediction_lead_time_for_fit,run_num,out_file);
  plot_IC_and_max_anomaly_in_PC_space(use_THC_norm_kernel,out_file,run_num)
end

% plot contours of optimal initial condition and maximum amplified
% solution of T,S:
if do_plot_optimal_ic
  use_THC_norm_kernel=0;
  plot_optimal_ic(use_THC_norm_kernel,run_num,out_file)
  use_THC_norm_kernel=1;
  plot_optimal_ic(use_THC_norm_kernel,run_num,out_file)
end

% plot stages during an oscillation predicted by the model in both
% real and EOF space, starting from optimal initial conditions:
if do_plot_linear_model_oscillation
  use_THC_norm_kernel=1;
  plot_linear_model_oscillation(n_PCs_to_fit,prediction_lead_time_for_fit,tau_max,use_THC_norm_kernel,run_num,out_file);
  use_THC_norm_kernel=0;
  plot_linear_model_oscillation(n_PCs_to_fit,prediction_lead_time_for_fit,tau_max,use_THC_norm_kernel,run_num,out_file);
end

% profile off; mkdir('profile_results'); profsave(profile('info'),'profile_results')

fprintf(out_file,'optimal_THC_IC_by_linear_fit_2GFDL_CM2 is done.\n');

close all; clear all;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [T,S,x,y,z,mask,mask3d,nt,nx,ny,nz]=...
      read_and_subsample_data(do_read_raw_netcdf_data,do_read_TS_data ...
              ,ncenturies, do_contour_EOFs, read_mask, print_raw_mask ...
              ,do_contour_TS, data_dir,netcdf_path,do_smooth_data_in_time ...
              ,do_avg_instead_of_subsample ...
              ,subsample_x,subsample_y,subsample_z,subsample_t,nsmooth_passes ...
              ,nx_b,nx_e,ny_b,ny_e,nz_b,nz_e,nt_b,nt_e,t_run_avg,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% netcdf programming tip: To look at netcdf files with a graphics
% interface, just type "ncbrowser" in matlab after the above path was
% set.

% also, to find out which variables are in a netcdf file, use:
% $ ncdump file.nc | more
% and look at the header of the ascii output.  

if do_read_raw_netcdf_data
  fprintf(out_file,'reading raw netcdf data...\n');
  if do_avg_instead_of_subsample
    fprintf(out_file,'averaging from fine to coarse grid (time consuming)...\n');
  else
    fprintf(out_file,'subsampling from fine to coarse grid...\n');
  end

  % read data for all years, put it into arrays T and S, and save t
  % mat file for future use:

  for icentury=1:ncenturies
    % netcdf file name:
    year_string=sprintf('%4.4d-%4.4d',1+100*(icentury-1),icentury*100);
    fnm_T=[data_dir 'ocean.' year_string '.temp.nc'];
    fnm_S=[data_dir 'ocean.' year_string '.salt.nc'];
    %  Open the NetCDF file
    fprintf(out_file,'century=%s\n',year_string);
    fnc_T = netcdf(fnm_T, 'nowrite');
    fnc_S = netcdf(fnm_S, 'nowrite');
    if isempty(fnc_T) || isempty(fnc_S); error('## Bad netcdf operation.'); end
    
    xt_ocean=ncvar('xt_ocean',fnc_T);
    if isempty(xt_ocean), error(' ## xt_ocean: No such variable.'), end  
    yt_ocean=ncvar('yt_ocean',fnc_T);
    if isempty(yt_ocean), error(' ## yt_ocean: No such variable.'), end  
    zt_ocean=ncvar('zt_ocean',fnc_T);
    if isempty(zt_ocean), error(' ## zt_ocean: No such variable.'), end  
    T_tmp=ncvar('temp',fnc_T);
    if isempty(T_tmp), error(' ## temp: No such variable.'), end  
    S_tmp=ncvar('salt',fnc_S);
    if isempty(S_tmp), error(' ## salt: No such variable.'), end  
    
    % get dimensions of data:
    dim=size(T_tmp);nt_century=dim(1);
    nz_full=dim(2);ny_full=dim(3);nx_full=dim(4);
    
    if read_mask
      read_mask=0;
      if print_raw_mask; 
        disp('*** cannot have both read_mask & print_raw_mask be 1.');
        quit;
      end
      fprintf(out_file,'reading mask... ');
      mask_tmp=load('mask.dat');
      mask_full=zeros(ny_full,nx_full);
      for jj=1:ny_full
        mask_full(ny_full-jj+1,:)=mask_tmp(jj,:);
      end
      % set mask to NaN except in Atlantic Ocean:
      mask_full(mask_full~=2)=NaN; mask_full(mask_full==2)=1;
    
      % mask on subsampled grid:
      mask=mask_full(ny_b:subsample_y:ny_e,nx_b:subsample_x:nx_e);
      clear mask_tmp mask_full
    
      fprintf(out_file,' done.\n');
    end
  
    if icentury==1;
      % stuff that needs to be done only once, do it during 1st century:
    
      % setup the arrays for storing temperature and salinity for
      % all times in the entire atlantic ocean based on dimensions
      % read from netcdf file and specified subsampling intervals:
      nx_sub=length(nx_b:subsample_x:nx_e);
      ny_sub=length(ny_b:subsample_y:ny_e);
      nz_sub=length(nz_b:subsample_z:nz_e);
      T=zeros(length(nt_b:nt_e),nz_sub,ny_sub,nx_sub);
      S=zeros(length(nt_b:nt_e),nz_sub,ny_sub,nx_sub);
    
      % print mask of zero over land and one over ocean to be
      % edited by hand to indicate where each ocean basin is:
      if print_raw_mask
        fprintf(out_file,'printing raw mask...\n');
        fid_mask = fopen('mask_raw.dat','w');
        mask=squeeze(T_tmp(1,1,:,:));
        mask(mask<-10000)=NaN;
        mask(not(isnan(mask)))=1;
        mask(isnan(mask))=0;
        for jj=ny:-1:1
          for ii=1:1:nx
            fprintf(fid_mask,'%1i',mask(jj,ii));
          end
          fprintf(fid_mask,'\n');
        end
        fclose(fid_mask);
        fprintf(out_file, ...
                'wrote mask file, Enter to quit, need to edit mask by hand now.\n');
        pause;
        return;
      end
    
      % setup axes for subsampled data:
      x=xt_ocean(nx_b:subsample_x:nx_e);
      y=yt_ocean(ny_b:subsample_y:ny_e);
      z=zt_ocean(nz_b:subsample_z:nz_e);
      
    end
    if do_avg_instead_of_subsample
      %% loop over coarse grid indices:
      for k=1:nz_sub
        for jj=1:ny_sub
          for ii=1:nx_sub
	    %% specify averaging range in fine grid indices:
            krange_s=max(nz_b,nz_b+(k -1)*subsample_z-floor(subsample_z/2));
            jrange_s=max(ny_b,ny_b+(jj-1)*subsample_y-floor(subsample_y/2));
            irange_s=max(nx_b,nx_b+(ii-1)*subsample_x-floor(subsample_x/2));
            krange_e=min(nz_e,nz_b+(k -1)*subsample_z+floor(subsample_z/2));
            jrange_e=min(ny_e,ny_b+(jj-1)*subsample_y+floor(subsample_y/2));
            irange_e=min(nx_e,nx_b+(ii-1)*subsample_x+floor(subsample_x/2));
	    krange=[krange_s:krange_e];
	    jrange=[jrange_s:jrange_e];
	    irange=[irange_s:irange_e];
	    %% average T,S from fine grid to coarse grid:
	    T(1+(icentury-1)*100:icentury*100,k,jj,ii)= ...
		squeeze(nanmean(nanmean(nanmean(T_tmp(:,krange,jrange,irange),4),3),2));
	    S(1+(icentury-1)*100:icentury*100,k,jj,ii)= ...
		squeeze(nanmean(nanmean(nanmean(S_tmp(:,krange,jrange,irange),4),3),2));
          end
        end
      end
    else
      %% subsample from fine to coarse grid:
      T(1+(icentury-1)*100:icentury*100,:,:,:)= ...
          T_tmp(:,nz_b:subsample_z:nz_e,ny_b:subsample_y:ny_e ...
                ,nx_b:subsample_x:nx_e);
      S(1+(icentury-1)*100:icentury*100,:,:,:)= ...
          S_tmp(:,nz_b:subsample_z:nz_e,ny_b:subsample_y:ny_e ...
                ,nx_b:subsample_x:nx_e);
    end
    
  end
  
  clear T_tmp S_tmp xt_ocean yt_ocean zt_ocean fnc_T fnc_S;
  
  % apply basin mask:
  fprintf(out_file,'applying mask to subsampled data...');
  % mask land points in 3d:
  T(T<-1000)=NaN;   S(S<-1000)=NaN; 
  % now set areas that are not within the atlantic to NaN, as defined by the
  % 2d mask:
  for ii=1:nx_sub
    for jj=1:ny_sub
      T(:,:,jj,ii)=T(:,:,jj,ii)*mask(jj,ii);
    end
  end
  fprintf(out_file,' done.\n');

  fprintf(out_file,['saving nonsmoothed non subsampled in time T,S data' ...
                    ' to file...']);
  dim_T=size(T); nt=dim_T(1); nz=dim_T(2); ny=dim_T(3); nx=dim_T(4);
  save(sprintf('Output/run%2.2d/atlantic_data_nonsmoothed_non_t_subsampled.mat',run_num) ...
       ,'T','S','x','y','z','nt','nx','ny','nz');
  fprintf(out_file,' done.\n');

  fprintf(out_file,'saving nonsmoothed subsampled T,S data to file...');
  % save original data to T_save, S_save, before subsampling and restore T,S
  % to their original structure after saving the non smoothed data and before
  % proceeding with the smoothing:
  T_save=T; S_save=S;
  T1=T(nt_b:subsample_t:nt_e,:,:,:);
  S1=S(nt_b:subsample_t:nt_e,:,:,:);
  clear T S; T=T1; S=S1; clear T1 S1;
  % revise dimensions of subsampled data after subsampling in time:
  dim_T=size(T); nt=dim_T(1); nz=dim_T(2); ny=dim_T(3); nx=dim_T(4);
  save(sprintf('Output/run%2.2d/atlantic_data_nonsmoothed.mat',run_num) ...
       ,'T','S','x','y','z','nt','nx','ny','nz');
  clear T S; T=T_save; S=S_save; clear T_save S_save;
  fprintf(out_file,' done.\n');

  % calculate 3d mask:
  mask3d=squeeze(T(1,:,:,:));
  mask3d(mask3d>-1000)=1; 
  mask3d(mask3d<-1000)=NaN; 
  save(sprintf('Output/run%2.2d/masks.mat',run_num),'mask','mask3d');
  
else % read_raw_netcdf_data

  fprintf(out_file,['loading nonsmoothed non sunsampled in time' ...
                    ' pre-processed T,S data...']);
  load(sprintf('Output/run%2.2d/atlantic_data_nonsmoothed_non_t_subsampled.mat',run_num));
  load(sprintf('Output/run%2.2d/masks.mat',run_num));
  fprintf(out_file,' done.\n');

end % read_raw_netcdf_data

% set dimensions of subsampled (in space) data:
dim_T=size(T); nt=dim_T(1); nz=dim_T(2); ny=dim_T(3); nx=dim_T(4);

if nsmooth_passes>0
  fprintf(out_file,'applying spatial smoothing to subsampled data...\n');
  fprintf(out_file,'nsmooth_passes=%d\n',nsmooth_passes);
  for ismooth_passes=1:nsmooth_passes
    tprint=nt/10;
    clear tmp1 tmp1 wgt;
    for t=1:nt
    
      %% print a progress indicator:
      if t==1; fprintf(out_file,'t=%d.. ',t);end
      if floor(t/tprint)*tprint==t; fprintf(out_file,'%d.. ',t);end
      if t==nt; fprintf(out_file,'\n');end
      T_smooth=zeros(nz,ny,nx)*NaN;
      S_smooth=zeros(nz,ny,nx)*NaN;
      for k=1:nz
        for jj=1:ny
          for ii=1:nx
            if not(isnan(T(t,k,jj,ii)))
              tmp1=7*T(t,k,jj,ii);
              tmp2=7*S(t,k,jj,ii);
              wgt=7;

	      % the following seems much slower and possibly not
              % debugged:
%	      wgt=wgt+sum(sum(not(isnan(T(t,k,max(1,jj-1):min(ny,jj+1),max(1,ii-1):min(nx,ii+1))))));
%	      tmp1=tmp1+nanmean(nanmean(T(t,k,max(1,jj-1):min(ny,jj+1),max(1,ii-1):min(nx,ii+1))));
%	      tmp2=tmp2+nanmean(nanmean(S(t,k,max(1,jj-1):min(ny,jj+1),max(1,ii-1):min(nx,ii+1))));

	      % so use this instead:
              for j1=max(1,jj-1):min(ny,jj+1)
                for i1=max(1,ii-1):min(nx,ii+1)
                  if not(isnan(T(t,k,j1,i1)))
                    tmp1=tmp1+T(t,k,j1,i1);
                    tmp2=tmp2+S(t,k,j1,i1);
                    wgt=wgt+1;
                  end
                end
              end

              T_smooth(k,jj,ii)=tmp1/wgt;
              S_smooth(k,jj,ii)=tmp2/wgt;
            end
          end
        end
      end
      T(t,:,:,:)=T_smooth;
      S(t,:,:,:)=S_smooth;
    end
  end
  fprintf(out_file,'      ... done spatial smoothing.\n');
end

if do_smooth_data_in_time
  fprintf(out_file,'applying running mean to subsampled data...\n');
  if round(t_run_avg)~=t_run_avg || t_run_avg<=0;
    fprintf(out_file,'*** expecting a positive integer t_run_avg\n');
    quit
  end
  clear tmp1 tmp1 wgt;
  tprint=nt/10;
  for t=1:nt
    
    % print a progress indicator:
    if t==1; fprintf(out_file,'t=%d.. ',t);end
    if floor(t/tprint)*tprint==t; fprintf(out_file,'%d.. ',t);end
    if t==nt; fprintf(out_file,'\n');end
    
    tmp1=zeros(nz,ny,nx); tmp2=zeros(nz,ny,nx);
    tot_wgt=0;
    for tt=max(t-2*t_run_avg,1):min(nt,t+2*t_run_avg);
      wgt=exp(-(t-tt)^2/t_run_avg^2);
      tot_wgt=tot_wgt+wgt;
      tmp1=tmp1+wgt*squeeze(T(tt,:,:,:));
      tmp2=tmp2+wgt*squeeze(S(tt,:,:,:));
    end
    T(t,:,:,:)=tmp1/tot_wgt;
    S(t,:,:,:)=tmp2/tot_wgt;
  end
  
  clear tmp1 tmp2;
  fprintf(out_file,'      ... done.\n');
else
  fprintf(out_file,'not smoothing in time.\n');
end

fprintf(out_file,'subsampling in time...');
T1=T(nt_b:subsample_t:nt_e,:,:,:);
S1=S(nt_b:subsample_t:nt_e,:,:,:);
clear T S;
T=T1; S=S1;
clear T1 S1;
% revise dimensions of subsampled data after subsampling in time:
dim_T=size(T); nt=dim_T(1); nz=dim_T(2); ny=dim_T(3); nx=dim_T(4);
fprintf(out_file,' done.\n');

fprintf(out_file,'saving smoothed T,S data to file...');
save(sprintf('Output/run%2.2d/atlantic_data_smoothed.mat',run_num) ...
     ,'T','S','x','y','z','nt','nx','ny','nz');
fprintf(out_file,' done.\n');

fprintf(out_file,'vertical levels: \n'); 
fprintf(out_file,'     %g, %g, %g, %g, %g\n',z); fprintf(out_file,'\n');
fprintf(out_file,'dimension of data: nt=%d, nz=%d, ny=%d, nx=%d\n',nt,nz,ny,nx);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function contour_TS(out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(out_file,'contouring temperature and salinity...\n');

load(sprintf('Output/run%2.2d/atlantic_data_nonsmoothed_non_t_subsampled.mat',run_num));
load(sprintf('Output/run%2.2d/masks.mat',run_num));
load(sprintf('Output/run%2.2d/T_avg_and_std.mat',run_num));

% set dimensions of subsampled (in space) data:
dim_T=size(T); nt=dim_T(1); nz=dim_T(2); ny=dim_T(3); nx=dim_T(4);
icentury=6;
n_years_to_plot=20;
plot_every_yrs=2;
fprintf(out_file,' start yr: %d, plot every %d yrs; total of %d yrs.\n' ...
	,(icentury-1)*100+1,plot_every_yrs,n_years_to_plot);
% put data in plot arrays:
for tt=1:n_years_to_plot
  % plot every two years:
  T_plot(tt,:,:,:)=squeeze(T((icentury-1)*100+tt*plot_every_yrs,:,:,:));
  S_plot(tt,:,:,:)=squeeze(S((icentury-1)*100+tt*plot_every_yrs,:,:,:));
  T_anom_plot(tt,:,:,:)=squeeze(T((icentury-1)*100+tt*plot_every_yrs,:,:,:))-T_avg;
  S_anom_plot(tt,:,:,:)=squeeze(S((icentury-1)*100+tt*plot_every_yrs,:,:,:))-S_avg;
end

% call plotting routine:
plot_eofs_what_to_contour='TS';
plot_3dTS(T_plot,S_plot,x,y,z,mask3d,n_years_to_plot,nx,ny,nz ...
	  ,plot_eofs_what_to_contour,out_file,run_num);

plot_eofs_what_to_contour='TS-anom';
plot_3dTS(T_anom_plot,S_anom_plot,x,y,z,mask3d,n_years_to_plot,nx,ny,nz ...
	  ,plot_eofs_what_to_contour,out_file,run_num);

fprintf(out_file,'done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [im,jm,km,nsea]=rearrange_TS_into_vector_X ...
    (T,S,x,y,z,mask,nt,nx,ny,nz,use_smoothed_TS_data ...
     ,do_apply_grid_weights,do_normalize_by_std,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the autocorrelation and EOFs are calculated using the smoothed T,S
% data.  Still, later the non smoothed data must be projected on these
% EOFs.  This routine is therefore called twice, do save smoothed
% and nonsmoothed versions of X.

if use_smoothed_TS_data
  fprintf(out_file,'putting smoothed T,S data into a vector...');
else
  fprintf(out_file,'putting nonsmoothed T,S data into a vector...');
end

% find indices of sea points in the data (Matlab's find function seems
% not to work with 3D arrays, so I have to do this myself):
n=0;
for ii=1:nx
  for jj=1:ny
    for k=1:nz
      if not(isnan(T(1,k,jj,ii)));
        n=n+1;
        im(n)=ii; jm(n)=jj; km(n)=k;
      end
    end
  end
end
nsea=n;

% put all T,S points at a given time into one large vector, do
% for all times:
X=zeros(nt,nsea*2);
for t=1:nt
  for n=1:nsea
    X(t,n)      = T(t,km(n),jm(n),im(n));
    X(t,n+nsea) = S(t,km(n),jm(n),im(n));
  end
end
fprintf(out_file,' done.\n');
fprintf(out_file,'number of sea points: %d\n',nsea);

if not(use_smoothed_TS_data)
  fprintf(out_file,'saving non-smoothed dimensional, but detrended X ...');
  X_nonsmoothed_detrended_dimensional=zeros(size(X));
  for n=1:nsea*2
    X_nonsmoothed_detrended_dimensional(:,n)=detrend(X(:,n));
  end
  save(sprintf('Output/run%2.2d/X_nonsmoothed_detrended_dimensional',run_num) ...
       ,'X_nonsmoothed_detrended_dimensional')
  clear X_nonsmoothed_detrended_dimensional;
  fprintf(out_file,' done.\n');
end


fprintf(out_file,'calculating grid-weights...');
% calculate weights based on grid box volume:
% first, calculate dx,dy, dz:
%% this assumes that levels start at z=0, the surface:
dz(1)=(z(2)+z(1))/2; 
for k=2:nz-1
  dz(k)=abs(z(k+1)-z(k-1))/2;
end
dz(nz)=abs(z(nz)-z(nz-1));

% cannot assume anything regarding the starting latitude:
dy(1)=(y(2)-y(1)); 
for k=2:ny-1
  dy(k)=abs(y(k+1)-y(k-1))/2;
end
dy(ny)=abs(y(ny)-y(ny-1));

for k=1:nx
  dx(k)=abs(x(2)-x(1)); % x-resolution is constant
end
  
% now weights:
if do_apply_grid_weights
  dv0=dz(1)*dy(1)*dx(1)*cos(y(round(ny/2))*pi/180);
  for k=1:nz
    for jj=1:ny
      for ii=1:nx
        grid_weights(k,jj,ii)=dz(k)*dy(jj)*dx(ii)*cos(y(jj)*pi/180)/dv0;
      end
    end
  end
else
  fprintf(out_file,'setting grid-weights to one...');
  for k=1:nz
    for jj=1:ny
      for ii=1:nx
        grid_weights(k,jj,ii)=1;
      end
    end
  end
end

% arrange weights into a single vector, like T,S:
X_grid_weights=zeros(nsea*2);
for n=1:nsea
  X_grid_weights(n)      = grid_weights(km(n),jm(n),im(n));
  X_grid_weights(n+nsea) = grid_weights(km(n),jm(n),im(n));
end
fprintf(out_file,' done.\n');

fprintf(out_file,'removing mean&linear trend from X data and scaling by variance...');
% also calculate mean and std and save them for future use: note that
% the detrend command also removes linear trend, not just mean:
for n=1:nsea*2
  X_avg(n)=mean(X(:,n));
  X(:,n)=detrend(X(:,n));
  X_std(n)=std(X(:,n));
  if do_normalize_by_std
    X(:,n)=X(:,n)/X_std(n);
  end
  X(:,n)=X(:,n)*X_grid_weights(n);
end
fprintf(out_file,' done.\n');

if use_smoothed_TS_data
  fprintf(out_file,'saving smoothed X...');
  save(sprintf('Output/run%2.2d/X_avg_and_std_smoothed',run_num),'X_avg','X_std')
  save(sprintf('Output/run%2.2d/X_smoothed_data.mat',run_num),'X','nt');
else
  fprintf(out_file,'saving nonsmoothed X...');
  save(sprintf('Output/run%2.2d/X_avg_and_std_nonsmoothed',run_num),'X_avg','X_std')
  save(sprintf('Output/run%2.2d/X_nonsmoothed_data.mat',run_num),'X','nt');
end
clear X X_avg X_std
save(sprintf('Output/run%2.2d/grid_weights.mat',run_num) ...
     ,'X_grid_weights','dx','dy','dz','grid_weights');
clear X_grid_weights dx dy dz grid_weights
fprintf(out_file,' done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function C=calc_autocorrelation(X,nt,out_file);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% form autocorrelation matrix
fprintf(out_file,'forming autocorrelation matrix...');
C = X'*X/(nt-1);
size_C=size(C); fprintf(out_file,'dimension(C)=%dx%d\n',size(C));
fprintf(out_file,' done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [EOFs_T,EOFs_S]=calculate_EOFs(n_EOFs,C,im,jm,km,nsea ...
		    ,x,y,z,nx,ny,nz,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% perform singular value decomposition:
fprintf(out_file,'calculating eigenvectors of autocorrelation...\n');
opts.disp=2;
[V,D,flag] = eigs(C,n_EOFs,'LM',opts);
if flag~=0;
  fprintf(out_file,'*** calculate_EOFs: eigenvalues did not converge!!***\n');
  return
end
clear C
fprintf(out_file,' ... done calculating eigenvalues/ vectors.\n');

fprintf(out_file,'eigenvalues: \n');
fprintf(out_file,'     %g %g %g %g %g %g %g %g \n',diag(D));
fprintf(out_file,'\n');

% calculate variance explained by EOF modes:
l=diag(D);
frac_variance = l / sum(l);
accum_frac_var=frac_variance;
for ii=2:size(accum_frac_var);
  accum_frac_var(ii)=accum_frac_var(ii-1)+ ...
      accum_frac_var(ii);
end
fprintf(out_file,'fraction of variance of T,S in each mode=\n');
fprintf(out_file,'   %6.2g %6.2g %6.2g %6.2g %6.2g %6.2g %6.2g\n',frac_variance); 
fprintf(out_file,'\n');
fprintf(out_file,'accumulated frac variance of T,S in each mode=\n');
fprintf(out_file,'   %6.2g %6.2g %6.2g %6.2g %6.2g %6.2g %6.2g\n',accum_frac_var); 
fprintf(out_file,'\n');
  
fprintf(out_file,['arranging eofs in 3d arrays EOFs_T and EOFs_S and' ...
                  ' dimensionalizing by std...']);
% arrange EOFs in a three dimensional array for plotting purposes,
load(sprintf('Output/run%2.2d/X_avg_and_std_nonsmoothed',run_num))
EOFs_T=zeros(n_EOFs,nz,ny,nx);
EOFs_S=zeros(n_EOFs,nz,ny,nx);

T_avg=zeros(nz,ny,nx);S_avg=zeros(nz,ny,nx);
T_std=zeros(nz,ny,nx);S_std=zeros(nz,ny,nx);
for e=1:n_EOFs
  for n=1:nsea
    EOFs_T(e,km(n),jm(n),im(n))=V(n,e);
    EOFs_S(e,km(n),jm(n),im(n))=V(n+nsea,e);
  end
end
for n=1:nsea
  T_avg(km(n),jm(n),im(n))=X_avg(n);
  S_avg(km(n),jm(n),im(n))=X_avg(n+nsea);
  T_std(km(n),jm(n),im(n))=X_std(n);
  S_std(km(n),jm(n),im(n))=X_std(n+nsea);
end

% save eigenvectors and eigenvalues:
save(sprintf('Output/run%2.2d/C_eigenvectors.mat',run_num),'V','l');
% save non dimensional EOFs on the physical 3d grid:
save(sprintf('Output/run%2.2d/EOFs.mat',run_num),'EOFs_T','EOFs_S','D','n_EOFs' ...
     ,'x','y','z','nx','ny','nz')
% save avg and std of T,S on the physical 3d grid:
save(sprintf('Output/run%2.2d/T_avg_and_std.mat',run_num) ...
     ,'T_avg','S_avg','T_std','S_std');
fprintf(out_file,' done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dimensionalize_EOFs(do_normalize_by_std,do_apply_grid_weights,out_file,run_num);;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate dimensinalization factors for each EOF for T,S, apply to
% EOFs, and save them:

fprintf(out_file,['calculating dimensionalization weights for T,S' ...
		  ' and dimensionalize EOFs...']);

load(sprintf('Output/run%2.2d/C_eigenvectors.mat',run_num));
load(sprintf('Output/run%2.2d/X_nonsmoothed_detrended_dimensional',run_num))
load(sprintf('Output/run%2.2d/PCs.mat',run_num));
load(sprintf('Output/run%2.2d/EOFs.mat',run_num));
load(sprintf('Output/run%2.2d/im_jm_km.mat',run_num));
load(sprintf('Output/run%2.2d/grid_weights.mat',run_num));
load(sprintf('Output/run%2.2d/X_avg_and_std_smoothed',run_num));

X_W_dimensionalize=zeros(nsea*2,n_EOFs);
TT=length(X_nonsmoothed_detrended_dimensional(:,1));

% use expression derived using SVD of the "data matrix" a-la Peter
% Huybers:
for e=1:n_EOFs
  for n=1:nsea*2
    %%XX something is still wrong with the dimensionalization
    X_W_dimensionalize(n,e) = sqrt(l(e));
    %XX X_W_dimensionalize(n,e) = sqrt(TT-1)*X_W_dimensionalize(n,e);
    if do_normalize_by_std
      X_W_dimensionalize(n,e) = X_W_dimensionalize(n,e)*X_std(n);
    end
    if do_apply_grid_weights
      X_W_dimensionalize(n,e) = X_W_dimensionalize(n,e)/X_grid_weights(n);
    end
  end
end

% Create 3d arrays to dimensionalize each EOF:
W_T_dimensionalize=zeros(n_EOFs,nz,ny,nx);
W_S_dimensionalize=zeros(n_EOFs,nz,ny,nx);
for e=1:n_EOFs
  for n=1:nsea
    W_T_dimensionalize(e,km(n),jm(n),im(n))=X_W_dimensionalize(n,e);
    W_S_dimensionalize(e,km(n),jm(n),im(n))=X_W_dimensionalize(n+nsea,e);
  end
end

for e=1:n_EOFs
  EOFs_T(e,:,:,:)=EOFs_T(e,:,:,:).*W_T_dimensionalize(e,:,:,:);
  EOFs_S(e,:,:,:)=EOFs_S(e,:,:,:).*W_S_dimensionalize(e,:,:,:);
end

save(sprintf('Output/run%2.2d/W_dimensionalize.mat',run_num) ...
     ,'W_T_dimensionalize','W_S_dimensionalize','X_W_dimensionalize');
save(sprintf('Output/run%2.2d/EOFs_dimensional.mat',run_num),'EOFs_T','EOFs_S','D','n_EOFs' ...
     ,'x','y','z','nx','ny','nz')

clear X_nonsmoothed_detrended_dimensional X_W_dimensionalize EOFs_T ...
    EOFs_S V PC;

fprintf(out_file,' done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_3dTS(EOFs_T,EOFs_S,x,y,z,mask3d,nplots,nx,ny,nz ...
                   ,plot_eofs_what_to_contour,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Used to plot sections of 3d EOFs, weights and GCM's T,S.

if strcmp(plot_eofs_what_to_contour,'weights')
  what='W';
elseif strcmp(plot_eofs_what_to_contour,'eofs')
  what='EOF';
elseif strcmp(plot_eofs_what_to_contour,'TS')
  what='TS';
elseif strcmp(plot_eofs_what_to_contour,'TS-anom')
  what='TS-anom';
else
  display('*** wrong value for plot_eofs_what_to_contour ***');
  quit;
end
fprintf(out_file,'plotting %ss...\n',what);

% plot:
for e=1:nplots
  figure
  subplot(3,2,1);
  % T: apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(EOFs_T(e,:,:,:)),3);
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  title(sprintf('(a) %s %d, T',what,e));
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20N';'40N';'60N';'80N'});
  colorbar;
  
  subplot(3,2,2);
  % S: apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(EOFs_S(e,:,:,:)),3);
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  title(sprintf('(b) %s %d, S',what,e));
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20N';'40N';'60N';'80N'});
  colorbar;
  
  k=2; % vertical level at which plot is made
  subplot(3,2,3);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',10);
  hold on
  % T: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(EOFs_T(e,k,:,:));
  % contour:
  m_contourf(x,y,contour_data);
  % draw cost again due to a matlab bug that contours over land:
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  clear contour_data
  zz=round(z(k)); title(sprintf('(c) z=%gm, T',zz));
  colorbar;
  
  subplot(3,2,4);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',10);
  hold on
  % S: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(EOFs_S(e,k,:,:));
  % contour:
  m_contourf(x,y,contour_data);
  % draw cost again due to a matlab bug that contours over land:
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  clear contour_data
  zz=round(z(k)); title(sprintf('(d) z=%gm, S',zz));
  colorbar;
  
  k=10; % vertical level at which plot is made
  subplot(3,2,5);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',10);
  hold on
  % T: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(EOFs_T(e,k,:,:));
  % contour:
  m_contourf(x,y,contour_data);
  % draw cost again due to a matlab bug that contours over land:
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  clear contour_data
  zz=round(z(k)); title(sprintf('(e) z=%gm, T',zz));
  colorbar;
  
  subplot(3,2,6);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',10);
  hold on
  % S: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(EOFs_S(e,k,:,:));
  % contour:
  m_contourf(x,y,contour_data);
  % draw cost again due to a matlab bug that contours over land:
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  clear contour_data
  zz=round(z(k)); title(sprintf('(f) z=%gm, S',zz));
  colorbar;
  
  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/%s_%2.2d',run_num,what,e);
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
end

fprintf(out_file,'   ... done plotting %s.\n',what);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function PC=calc_principal_components ...
    (do_plot_PCs,do_calc_PCs_from_smoothed_data,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(out_file,'calculating principal components...');

if do_calc_PCs_from_smoothed_data
  fprintf(out_file,' from **smoothed** data...');
  load(sprintf('Output/run%2.2d/X_smoothed_data.mat',run_num));
else
  fprintf(out_file,' from nonsmoothed data...');
  load(sprintf('Output/run%2.2d/X_nonsmoothed_data.mat',run_num));
end
load(sprintf('Output/run%2.2d/C_eigenvectors.mat',run_num));

% calculate the principle componenets (time series);
n_EOFs=length(V(1,:));
for e=1:n_EOFs
  PC(:,e) = X(:,:)*V(:,e);
end
%size_X=size(X),size_V=size(V),size_PC=size(PC);

clear X V l;

if do_plot_PCs
  figure
  subplot(2,1,1);
  for e=n_EOFs:-1:1
    if e==1; clr='r'; else; clr='b'; end;  
    plot(PC(:,e),clr);
    hold on
    %fprintf(out_file,'enter for more PC plots\n'); pause
  end
  title('PC time series for temperature EOFs');
  xlabel('time');
  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/PCs_timeseries',run_num);
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
end
save(sprintf('Output/run%2.2d/PCs.mat',run_num),'PC');

% normalize PCs and save normalized version:
PC_n=0*PC;
for e=1:n_EOFs
  PC_n(:,e)=PC(:,e)-mean(PC(:,e));
  PC_n(:,e)=PC_n(:,e)/std(PC_n(:,e));
end
save(sprintf('Output/run%2.2d/PCs_normalized',run_num),'PC_n');

fprintf(out_file,' done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analyze_THC_index(out_file,do_filter_in_analyze_THC_index ...
			   ,do_use_normalized_PCs,which_THC_to_use ...
                           ,n_PCs_to_fit,t_run_avg,do_smooth_PCs,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read THC index from Data directory, and correlate with PC amplitudes

fprintf(out_file,'correlating THC index to EOFs...\n');

if do_use_normalized_PCs
  load(sprintf('Output/run%2.2d/PCs_normalized.mat',run_num));
else
  load(sprintf('Output/run%2.2d/PCs.mat',run_num));
  PC_n=PC; clear PC;
end
if do_smooth_PCs; PC=smooth_PCs(PC_n,t_run_avg,out_file); end

THC1=load('Data/ocean.0001-2000_THC_ATL_20N_0m_5000m.ascii');
THC2=load('Data/ocean.0001-2000_THC_ATL_20N_80N_0m_5000m.ascii');

% get only the final 1500 years where the drift is smaller:
t1=THC1(501:2000); t2=THC2(501:2000); clear THC1 THC2; THC1=t1; THC2=t2; 
THC1_dimensional=THC1-mean(THC1); THC2_dimensional=THC2-mean(THC2);

% normalize before analyzing correlations:
THC1=THC1-mean(THC1); THC1=THC1/std(THC1); 
THC2=THC2-mean(THC2); THC2=THC2/std(THC2);

if do_filter_in_analyze_THC_index
  fprintf(out_file,'filtering time series before analysing correlations.\n');
  % filter:
  window=5;
  myfilter=(exp(-([-window:window]/2).^2))/sum(exp(-([-window:window]/2).^2));

  THC1=filter(myfilter,1,THC1);
  THC2=filter(myfilter,1,THC2);
  for e=1:n_PCs_to_fit
    PC_n(:,e)=filter(myfilter,1,PC_n(:,e));
  end
end

time=1:1500;

% calculate regression coeffs from normalized PCs to THC index: trying
% to find THC1_PC_regression_coeffs that minimize
% THC1-THC1_PC_regression_coeff*PC_n for all times.  Use only
% n_PCs_to_fit PCs.  This implies an equation for the coeffs whose
% solution is
THC1_PC_regression_coeff=squeeze(PC_n(:,1:n_PCs_to_fit)\THC1_dimensional);
THC1_reconstructed=PC_n(:,1:n_PCs_to_fit)*THC1_PC_regression_coeff;
fprintf(out_file,'THC1 regression coeffs:\n');
fprintf(out_file,'     %g %g %g %g %g %g\n',THC1_PC_regression_coeff);
fprintf(out_file,'     \n');
THC2_PC_regression_coeff=squeeze(PC_n(:,1:n_PCs_to_fit)\THC2_dimensional);
THC2_reconstructed=PC_n(:,1:n_PCs_to_fit)*THC2_PC_regression_coeff;
fprintf(out_file,'THC2 regression coeffs:\n');
fprintf(out_file,'     %g %g %g %g %g %g\n',THC2_PC_regression_coeff);
fprintf(out_file,'     \n');

save(sprintf('Output/run%2.2d/THC_reconstructed.mat',run_num) ...
     ,'THC1_reconstructed','THC2_reconstructed');

% now plot the regression coeffs:
figure
axes('position',[0.1 0.3 0.5 0.3]);  % [left, bottom, width, height]
plot(THC1_PC_regression_coeff,'-x');
h(1)=xlabel(' coeff #');
h(2)=ylabel('regression coeff');
h(3)=gca;
set(h,'fontname','helvetica','fontsize',14,'fontweight','bold');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/THC1_PC_regression_coeffs',run_num);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

figure
axes('position',[0.1 0.3 0.5 0.3]);  % [left, bottom, width, height]
plot(THC2_PC_regression_coeff,'-x');
h(1)=xlabel(' coeff #');
h(2)=ylabel('regression coeff');
h(3)=gca;
set(h,'fontname','helvetica','fontsize',14,'fontweight','bold');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/THC2_PC_regression_coeffs',run_num);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

% calculate the norm kernel for the THC based on the regression coeffs:
if which_THC_to_use==1
  X_THC=THC1_PC_regression_coeff*THC1_PC_regression_coeff';
elseif which_THC_to_use==2
  X_THC=THC2_PC_regression_coeff*THC2_PC_regression_coeff';
else
  disp('error in which_THC_to_use');
  quit
end
norm_X_THC=norm(X_THC);
X_THC=X_THC/norm_X_THC;
regularization_coeff=1.0e-3;
X_THC=X_THC+regularization_coeff*eye(n_PCs_to_fit);

fprintf(out_file,'the dimension of the THC norm kernel matrix is: %dx%d\n',size(X_THC));
fprintf(out_file,'regularization_coeff=%d, norm(X_THC) before regularization=%d' ...
	,regularization_coeff,norm_X_THC);

save(sprintf('Output/run%2.2d/norm_kernel_and_normalized_THC_timeseries.mat',run_num) ...
     ,'X_THC' ...
     ,'THC1_PC_regression_coeff','THC2_PC_regression_coeff' ...
     ,'THC1','THC2' ...
     ,'THC1_dimensional','THC2_dimensional');

% plot original THC index, and that reconstructed by regression
% from PCs:
figure
axes('position',[0.1 0.3 0.8 0.3]);  % [left, bottom, width, height]
plot(time,THC1_dimensional,'r')
hold on
plot(time,THC1_reconstructed,'g')
%h(1)=title('r: original THC1; g: reconstructed from PC_n by regression');
h(2)=xlabel('time (yr)');
h(3)=ylabel('THC index');
h(4)=gca;
h(5)=legend('GCM','reconstructed');
set(h(1:4),'fontname','helvetica','fontsize',14,'fontweight','bold');
set(h(5),'fontname','helvetica','fontsize',12,'fontweight','normal');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/THC1-and-reconstructed-THC1',run_num);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

figure
axes('position',[0.1 0.3 0.8 0.3]);  % [left, bottom, width, height]
plot(time,THC2_dimensional,'r')
hold on
plot(time,THC2_reconstructed,'g')
%h(1)=title('r: original THC2; g: reconstructed from PC_n by regression');
h(2)=xlabel('time (yr)');
h(3)=ylabel('THC index');
h(4)=gca;
h(5)=legend('GCM','reconstructed');
set(h(1:4),'fontname','helvetica','fontsize',14,'fontweight','bold');
set(h(5),'fontname','helvetica','fontsize',12,'fontweight','normal');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/THC2-and-reconstructed-THC2',run_num);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

% calculate the lag correlation of the two THC indices with all PCs,
% find the lag of maximum correlation, and the value of the
% correlation at that lag as well as at zero lag:
n_positive_lags=30;
[corr_THC1_THC2,lags_THC1_THC2]=xcov(THC1,THC2,n_positive_lags,'coeff');
[c_THC1_THC2,nn]=max(abs(corr_THC1_THC2)); l_THC1_THC2=lags_THC1_THC2(nn);
corr_THC1_PCn=zeros(n_PCs_to_fit,2*n_positive_lags+1);
lags_THC1_PCn=zeros(n_PCs_to_fit,2*n_positive_lags+1);
for iPC=1:n_PCs_to_fit
  %% THC1:
  [corr_THC1_PCn(iPC,:),lags_THC1_PCn(iPC,:)]=xcov(THC1,PC_n(:,iPC),n_positive_lags,'coeff');
  [c_THC1_PCn(iPC),nn]=max(abs(corr_THC1_PCn(iPC,:))); l_THC1_PCn(iPC)=lags_THC1_PCn(iPC,nn);
  c_THC1_PCn_zero_lag(iPC)=corr_THC1_PCn(iPC,n_positive_lags+1);
  
  %% THC2:
  [corr_THC2_PCn(iPC,:),lags_THC2_PCn(iPC,:)]=xcov(THC2,PC_n(:,iPC),n_positive_lags,'coeff');
  [c_THC2_PCn(iPC),nn]=max(abs(corr_THC2_PCn(iPC,:))); l_THC2_PCn(iPC)=lags_THC2_PCn(iPC,nn);
  c_THC2_PCn_zero_lag(iPC)=corr_THC2_PCn(iPC,n_PCs_to_fit+1);
end
[corr1,lags1]=xcov(THC1,THC1_reconstructed,n_positive_lags,'coeff');
[B,IX]=sort(abs(corr1)); nn=IX(length(B)); c1=corr1(nn); l1=lags1(nn);
[corr2,lags2]=xcov(THC2,THC2_reconstructed,n_positive_lags,'coeff');
[B,IX]=sort(abs(corr2)); nn=IX(length(B)); c2=corr2(nn); l2=lags2(nn);

% print various correlations:
fprintf(out_file,[' corr(THC1,THC2)=%g, at lag=%d\n'...
           ,' corr(THC1,THC1_reconstructed)=%g, at lag=%d\n' ...
           ,' corr(THC2,THC2_reconstructed)=%g, at lag=%d\n' ...
                 ] ...
           ,c_THC1_THC2,l_THC1_THC2,c1,l1,c2,l2);
for iPC=1:n_PCs_to_fit
  fprintf(out_file,[' corr(THC1,PC%d)=%g, at lag=%d\t' ...
		    ' corr(THC1,PC%d)=%g, at lag=0 \n' ... 
		    ' corr(THC2,PC%d)=%g, at lag=%d\t' ...
		    ' corr(THC2,PC%d)=%g, at lag=0 \n' ...
		   ] ...
	  ,iPC,c_THC1_PCn(iPC),l_THC1_PCn(iPC) ...
	  ,iPC,c_THC1_PCn_zero_lag(iPC) ...
	  ,iPC,c_THC2_PCn(iPC),l_THC2_PCn(iPC) ...
	  ,iPC,c_THC2_PCn_zero_lag(iPC));
end

% now plot the correlation coeffs at zero lag:
figure
axes('position',[0.1 0.3 0.5 0.3]);  % [left, bottom, width, height]
plot(c_THC1_PCn_zero_lag,'k-');
h(1)=xlabel('PC #');
h(2)=ylabel('corr coeff');
h(3)=gca;
set(h(1:3),'fontname','helvetica','fontsize',14,'fontweight','bold');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/THC1_PC_correlation_coeffs_zero_lag',run_num);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

figure
axes('position',[0.1 0.3 0.5 0.3]);  % [left, bottom, width, height]
plot(c_THC2_PCn_zero_lag,'k-');
h(1)=xlabel('PC #');
h(2)=ylabel('corr coeff');
h(3)=gca;
set(h(1:3),'fontname','helvetica','fontsize',14,'fontweight','bold');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/THC2_PC_correlation_coeffs_zero_lag',run_num);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

fprintf(out_file,' ... done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function A=fit_linear_model(n_PCs_to_fit,prediction_lead_time_for_fit ...
   ,do_use_normalized_PCs,do_smooth_PCs ...
   ,N_seg,t_fit_start,t_fit_end,plot_in_fit_linear_model,fit_AR1,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fit a linear model (matrix) to the evolution of the PC state vector.
% input: PC: observed PC time series from GCM run;  n_PCs_to_fit:
% as name implies; prediction_lead_time_for_fit=same;
% t_fit_start,t_fit_end: these could be vectors of length N_seg that specify a
% number of segments of the time series over which the fit should
% be performed.
if fit_AR1==1
  fprintf(out_file,'calculating an AR1 linear fit to the PC evolution...\n');
else
  fprintf(out_file,'calculating a linear fit to the PC evolution...\n');
end

if do_use_normalized_PCs
  load(sprintf('Output/run%2.2d/PCs_normalized.mat',run_num));
else
  load(sprintf('Output/run%2.2d/PCs.mat',run_num));
  PC_n=PC; clear PC;
end
PC=PC_n; clear PC_n;
if do_smooth_PCs; PC_n=smooth_PCs(PC_n,t_run_avg,out_file); end

% The principle components are a function of space (i) and time (n), so can
% be written as a matrix PC(n,i) whose dimensions are:
I=length(PC(1,:));
N=0;
for nn=1:N_seg;
  N=N+length(t_fit_start(nn):t_fit_end(nn));
end

% but let's try to fit a model to less EOFs first:
I=n_PCs_to_fit;
fprintf(out_file,['   using %d principle components,\n'...
           '       %d elements in the PC time series,\n'...
           '   in %d segments.\n'...
           '   start&end times: %g %g, %g %g'] ...
           ,  I,N,N_seg,t_fit_start,t_fit_end);
fprintf(out_file,'\n');

% Let A=a_ij be the matrix we are looking for.  P^n_j is the j'th element of
% the PC at time n.  (i,j)=1...I and n=1...N.  The equation for the a_ij is:
  
% ---
% a_11*P^n_1+...+a_1I*P^n_I=P^n+1_1
% a_21*P^n_1+...+a_2I*P^n_I=P^n+1_2
% ...
% a_I1*P^n_1+...+a_II*P^n_I=P^n+1_I
% ---
% a_11*P^n+1_1+...+a_1I*P^n+1_I=P^n+2_1
% a_21*P^n+1_1+...+a_2I*P^n+1_I=P^n+2_2
% ...
% a_I1*P^n+1_1+...+a_II*P^n+1_I=P^n+2_I
% ---
% ...  
% ---
% a_11*P^N-1_1+...+a_1I*P^N-1_I=P^N_1
% a_21*P^N-1_1+...+a_2I*P^N-1_I=P^N_2
% ...
% a_I1*P^N-1_1+...+a_II*P^N-1_I=P^N_I
% ---

% This needs to be rewritten as an equation in which the a_ij form a
% vector:
% G(NxI,IxI)*A_vec(IxI,1)=b(NxI,1) 

% Start by letting the rhs vector in the above equations be
b=zeros(I*(N-prediction_lead_time_for_fit),1);
for ii=1:I
  n=0;
  for n_segment=1:N_seg
    for nn=t_fit_start(n_segment):(t_fit_end(n_segment)-prediction_lead_time_for_fit);
      n=n+1;
      b((n-1)*I+ii)=PC(nn+prediction_lead_time_for_fit,ii);
    end
  end
end

% The matrix of coeffs in the equation for the a_ij is
G=zeros((N-prediction_lead_time_for_fit)*I,I*I);
for ii=1:I
  for jj=1:I
    n=0;
    for n_segment=1:N_seg
      for nn=t_fit_start(n_segment):(t_fit_end(n_segment)-prediction_lead_time_for_fit);
        n=n+1;
        G((n-1)*I+ii,(ii-1)*I+jj)=PC(nn,jj);
        if fit_AR1==1;
          if ii~=jj; G((n-1)*I+ii,(ii-1)*I+jj)=0; end;
        end
      end
    end
  end
end

% some diagnostics about the matrix equation to be solved:
sigma=svd(G,0);
if plot_in_fit_linear_model;
  plot(sigma);
  title('singular values of G');
end

fprintf(out_file,'   first and last singular values of G:\n');
fprintf(out_file,'   %g %g %g %g,..., %g %g %g %g\n',sigma(1:4),sigma(I*I-3:I*I));
cond_GTG=cond(G'*G);
fprintf(out_file,'   rank(G^TG)=%d; cond(G^TG)=%d\n',rank(G'*G),cond_GTG);

% solve for the a_ij in a vector form:
% size_G=size(G),size_b=size(b);

% A_vec=G\b;
A_vec=pinv(G)*b;

% and reshape the a_ij back into a matrix form:
A=reshape(A_vec,[I,I]);

% if fitting an AR(1) process, make sure all diagnonal elements are less
% than one:
if fit_AR1==1
  for ii=1:I;
    if abs(A(ii,ii))>1;
      old=A(ii,ii); new=1; A(ii,ii)=new;
      fprintf(out_file,'*** setting a diagnonal element of AR1 from %d to %d.\n' ...
              ,old,new);
    end
  end
end

clear b G A_vec;

if plot_in_fit_linear_model;
  figure
  subplot(2,2,1);
  image(A,'CDataMapping','scaled');
  colorbar
  title('A');
  subplot(2,2,2);
  image(A'*A,'CDataMapping','scaled');
  colorbar
  title('A^TA');
  subplot(2,2,3);
  image(A*A','CDataMapping','scaled');
  colorbar
  title('AA^T');
  subplot(2,2,4);
  image(A'*A-A*A','CDataMapping','scaled');
  colorbar
  title('A^TA-AA^T');
  FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/A_linearized_matrix_images',run_num);
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

end

fprintf(out_file,'done.\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function evaluate_fit_quality(A,AR1,n_PCs_to_fit,prediction_lead_time_for_fit ...
    ,plot_in_fit_linear_model,tau_max,do_use_normalized_PCs,do_smooth_PCs ...
    ,plot_obs_and_predicted_PCs,smooth_b4_plotting_predicted_PCs ...
    ,which_THC_to_use ...
    ,prediction_start_interval,do_tau_test,out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate residuals of linear model and print some diagnostics

fprintf(out_file,'evaluating quality of linear fit to the PC evolution...\n');
% get THC time series and regression coeffs to normalized PCs:
load(sprintf('Output/run%2.2d/norm_kernel_and_normalized_THC_timeseries.mat',run_num));

if do_use_normalized_PCs
  load(sprintf('Output/run%2.2d/PCs_normalized.mat',run_num));
else
  load(sprintf('Output/run%2.2d/PCs.mat',run_num));
  PC_n=PC; clear PC;
end
if do_smooth_PCs; PC_n=smooth_PCs(PC_n,t_run_avg,out_file); end

% The principle components are a function of space (i) and time (n),
% so can be written as a matrix PC(n,i).
I=n_PCs_to_fit;
N=length(PC_n(:,1));

% get a visual indication of how useful the linear matrix is:
if plot_obs_and_predicted_PCs
  
  if smooth_b4_plotting_predicted_PCs
    % filter time series before plotting prediction estimates vs data:
    fprintf(out_file,'***beware! smoothing time series before verification!\n');
    windowSize=40;
    for e=1:I
      PC_n(:,e)=filter(ones(1,windowSize)/windowSize,1,PC_n(:,e));
    end
  end

  % calculate A^p and AR1^p for all needed p values in advance, in
  % order to save computations below:
  Ap_saved=zeros(tau_max,I,I);
  AR1p_saved=zeros(tau_max,I,I);
  diagnostics=1;
  for tau=1:tau_max
    p=(tau/prediction_lead_time_for_fit);
    Ap_saved(tau,:,:)=MyMatrixPower(A,p,diagnostics,out_file);
    % eliminate all off diagnonal roundoff terms before raising
    % to power p to prevent soem funny matlab problem:
    AR1p_saved(tau,:,:)=MyMatrixPower(diag(diag(AR1)),p,diagnostics,out_file);
    t1=max(max(abs(imag(Ap_saved(tau,:,:)))));
    t2=max(max(abs(imag(AR1p_saved(tau,:,:)))));
    if t1>1.e-12 || t2>1.e-12;
      fprintf(out_file,'*** imag(Ap,AR1p) (%d, %d) too large!\n',t1,t2);return;
    end
  end
  diagnostics=0;
  
  % compared predicted and obsrved time series: start a prediction every
  % prediction_start_interval, and predict for lead times 1 to
  % tau_max.  plot these prediction segments together with the actual
  % observed time series:
  Npredictions=...
      length(1:prediction_start_interval:1500-prediction_start_interval);
  PC_predicted=      zeros(tau_max+1,I,Npredictions)*NaN;
  PC_predicted_AR1=  zeros(tau_max+1,I,Npredictions)*NaN;
  THC1_predicted=     zeros(tau_max+1,Npredictions)*NaN;
  THC1_predicted_AR1= zeros(tau_max+1,Npredictions)*NaN;
  THC2_predicted=     zeros(tau_max+1,Npredictions)*NaN;
  THC2_predicted_AR1= zeros(tau_max+1,Npredictions)*NaN;
  time_predicted=    zeros(tau_max+1,Npredictions)*NaN;
  npredict=0;
  for start_time=1:prediction_start_interval:1500-prediction_start_interval
    npredict=npredict+1;
    nb=start_time;
    PC_predicted(1,1:I,npredict)=PC_n(nb,1:I);
    PC_predicted_AR1(1,1:I,npredict)=PC_n(nb,1:I);
    THC1_predicted(1,npredict)=THC1_dimensional(nb);
    THC1_predicted_AR1(1,npredict)=THC1_dimensional(nb);
    THC2_predicted(1,npredict)=THC2_dimensional(nb);
    THC2_predicted_AR1(1,npredict)=THC2_dimensional(nb);
    time_predicted(1,npredict)=nb;
    for tau=1:tau_max
      Ap=squeeze(Ap_saved(tau,:,:));
      AR1p=squeeze(AR1p_saved(tau,:,:));
      PC_predicted(1+tau,1:I,npredict)=PC_n(nb,1:I)*Ap;
      PC_predicted_AR1(1+tau,1:I,npredict)=PC_n(nb,1:I)*AR1p;
      THC1_predicted(1+tau,npredict)=PC_n(nb,1:I)*Ap*THC1_PC_regression_coeff;
      THC1_predicted_AR1(1+tau,npredict)=PC_n(nb,1:I)*AR1p*THC1_PC_regression_coeff;
      THC2_predicted(1+tau,npredict)=PC_n(nb,1:I)*Ap*THC2_PC_regression_coeff;
      THC2_predicted_AR1(1+tau,npredict)=PC_n(nb,1:I)*AR1p*THC2_PC_regression_coeff;
      time_predicted(1+tau,npredict)=start_time+tau;
    end
  end

  % plot observed, predicted and residual time series for several PCs:
  figure
  % plot predicted time series in different clors, to allow
  % distinguishing between them:
  line_patterns=['-g';'-b';'-m';'-c';'-y']; num_patterns=length(line_patterns(:,1));
  panels_per_page=1; nfigure=0;
  for ii=1:n_PCs_to_fit;
    ipanel=mod(ii,panels_per_page); if ipanel==0; ipanel=panels_per_page; end;
%   subplot(min(panels_per_page,n_PCs_to_fit),1,ipanel);
    subplot(3,1,1);
    h=plot(1:N,PC_n(:,ii),'-r'); set(h,'linewidth',2);
    hold on;
    for npredict=1:Npredictions
      ipattern=mod(npredict,num_patterns); if ipattern==0; ipattern=num_patterns; end
      h=plot(time_predicted(:,npredict) ...
             ,real(PC_predicted(:,ii,npredict)),line_patterns(ipattern,:));
%      h=plot(time_predicted(:,npredict) ...
%             ,real(PC_predicted_AR1(:,ii,npredict)),'-b');
    end
%    legend(sprintf('observed %d',ii) ...
%       ,sprintf('predicted %d',ii));% ,sprintf('predicted AR1 %d',ii));
    plot(1:N,zeros(N,1),'k');
    %h=plot(1:N,PC_n(:,ii),'-r'); set(h,'linewidth',2);
    Vax=axis; axis([0 1500 Vax(3) Vax(4)]);
    % title('GCM''s and predicted PC');
    xlabel('time (years)');
    ylabel(sprintf('PC %d',ii));
    V=axis;
    ymin=min(min(real(PC_predicted(:,ii,:))))*1.05;
    ymax=max(max(real(PC_predicted(:,ii,:))))*1.05;
    axis([V(1) V(2) ymin ymax]); % [XMIN XMAX YMIN YMAX]
    if mod(ii,panels_per_page)==0;
      nfigure=nfigure+1;
      FigureBaseFileName=sprintf(...
          'Output/run%2.2d/Figures/predicted_vs_observed_PCs_%d',run_num,nfigure);
      saveas(gcf,[FigureBaseFileName,'.fig']);
      print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
      if ii~=n_PCs_to_fit; figure; end
    end
  end


  % plot predicted vs observed THC:

  %% THC1:
  figure
  subplot(3,1,1);
  h=plot(1:N,THC1_dimensional(:),'-r'); set(h,'linewidth',2);
  hold on;
  b=1; e=10;
  for npredict=1:Npredictions
    ipattern=mod(npredict,num_patterns); if ipattern==0; ipattern=num_patterns; end
    h=plot(time_predicted(b:e,npredict) ...
	   ,real(THC1_predicted(b:e,npredict)),line_patterns(ipattern,:));
    %      h=plot(time_predicted(b:e,npredict) ...
    %             ,real(THC1_predicted_AR1(b:e,ii,npredict)),'-b');
  end
  plot(1:N,zeros(N,1),'k');
  Vax=axis; axis([0 1500 Vax(3) Vax(4)]);
  xlabel('time (years)');
  ylabel(sprintf('THC'));
  V=axis;
  ymin=min(min(real(THC1_predicted(:,:))))*1.05;
  ymax=max(max(real(THC1_predicted(:,:))))*1.05;
  % plot only for a part of the time series, otherwise figure is illegible:
  axis([500 1000 ymin ymax]); % [XMIN XMAX YMIN YMAX]
  FigureBaseFileName=sprintf(...
      'Output/run%2.2d/Figures/predicted_vs_observed_THC1',run_num);
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);


  %% THC2:
  figure
  subplot(3,1,1);
  h=plot(1:N,THC2_dimensional(:),'-r'); set(h,'linewidth',2);
  hold on;
  b=1; e=10;
  for npredict=1:Npredictions
    ipattern=mod(npredict,num_patterns); if ipattern==0; ipattern=num_patterns; end
    h=plot(time_predicted(b:e,npredict) ...
	   ,real(THC2_predicted(b:e,npredict)),line_patterns(ipattern,:));
    %      h=plot(time_predicted(b:e,npredict) ...
    %             ,real(THC2_predicted_AR1(b:e,ii,npredict)),'-b');
  end
  plot(1:N,zeros(N,1),'k');
  Vax=axis; axis([0 1500 Vax(3) Vax(4)]);
  xlabel('time (years)');
  ylabel(sprintf('THC'));
  V=axis;
  ymin=min(min(real(THC2_predicted(:,:))))*1.05;
  ymax=max(max(real(THC2_predicted(:,:))))*1.05;
  % plot only for a part of the time series, otherwise figure is illegible:
  axis([500 1000 ymin ymax]); % [XMIN XMAX YMIN YMAX]
  FigureBaseFileName=sprintf(...
      'Output/run%2.2d/Figures/predicted_vs_observed_THC2',run_num);
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
  
end

clear PC_predicted PC_predicted_AR1 residuals 



% -----------
% "tau test":
% -----------
if do_tau_test
  % load normalized THC timeseries and THC to PC regression coeffs:
  load(sprintf('Output/run%2.2d/norm_kernel_and_normalized_THC_timeseries.mat',run_num));
  if do_use_normalized_PCs
    load(sprintf('Output/run%2.2d/PCs_normalized.mat',run_num));
  else
    load(sprintf('Output/run%2.2d/PCs.mat',run_num));
    PC_n=PC; clear PC;
  end
  ncenturies_tau_test=15;

  % Calculate variance of predicted minus observed as function of
  % lead time and compare to formula based on variance:
  % This requires using one century for validation after using all
  % other centuries for fitting the linear matrix.

  %% (1) fit a linear model to all cnturies except one, repeat for
  %%     all centuries and store the resulting matrices.  While doing
  %%     this, fit also an AR(1) (diagonal matrix) for comparison of
  %%     diagnostics later:
  PC_predicted=zeros(N,I)*NaN;
  THC1_predicted=zeros(N,1)*NaN;
  THC2_predicted=zeros(N,1)*NaN;
  residuals=zeros(N,I)*NaN;
  THC1_residuals=zeros(N,1)*NaN;
  THC2_residuals=zeros(N,1)*NaN;
  PC_predicted_AR1=zeros(N,I)*NaN;
  THC1_predicted_AR1=zeros(N,1)*NaN;
  THC2_predicted_AR1=zeros(N,1)*NaN;
  residuals_AR1=zeros(N,I)*NaN;
  THC1_residuals_AR1=zeros(N,1)*NaN;
  THC2_residuals_AR1=zeros(N,1)*NaN;
  A_all_centuries=zeros(ncenturies_tau_test,I,I);
  AR1_all_centuries=zeros(ncenturies_tau_test,I,I);
  for century=1:ncenturies_tau_test;
    % use all century but this one to evaluate the linear matrix, and
    % then this one for testing its quality:
    if     century==1;  N_seg=1;t_fit_start=[101];t_fit_end=[N];
    elseif century==ncenturies_tau_test; N_seg=1;t_fit_start=[1];  t_fit_end=[(century-1)*100];
    else; N_seg=2;t_fit_start=[1,century*100+1];t_fit_end=[(century-1)*100,N];
    end
    fprintf(out_file,' ----- \n');
    fprintf(out_file,'testing matrix on century %d after fit to all others\n',century);
    fit_AR1=0;
    A=fit_linear_model(n_PCs_to_fit,prediction_lead_time_for_fit ...
       ,do_use_normalized_PCs,do_smooth_PCs ...
       ,N_seg,t_fit_start,t_fit_end,plot_in_fit_linear_model,fit_AR1,out_file,run_num);
    fit_AR1=1;
    AR1=fit_linear_model(n_PCs_to_fit,prediction_lead_time_for_fit ...
       ,do_use_normalized_PCs,do_smooth_PCs ...
       ,N_seg,t_fit_start,t_fit_end,plot_in_fit_linear_model,fit_AR1,out_file,run_num);
    A_all_centuries(century,:,:)=A(:,:);
    AR1_all_centuries(century,:,:)=AR1(:,:);
  end

  %% (2) predict time series for that one century that was not used
  %%     for fitting the linear model:
  covar_prediction_error=zeros(tau_max,I,I);
  covar_prediction_error_AR1=zeros(tau_max,I,I);
  covar_data=squeeze(PC_n(:,1:I)'*PC_n(:,1:I))/(length(PC_n(:,1))-1);
  THC1_covar_prediction_error=zeros(tau_max,1);
  THC1_covar_prediction_error_AR1=zeros(tau_max,1);
  THC1_covar_data=squeeze(THC1_dimensional(:)'*THC1_dimensional(:)) ...
      /(length(THC1_dimensional(:))-1);
  THC2_covar_prediction_error=zeros(tau_max,1);
  THC2_covar_prediction_error_AR1=zeros(tau_max,1);
  THC2_covar_data=squeeze(THC2_dimensional(:)'*THC2_dimensional(:)) ...
      /(length(THC2_dimensional(:))-1);
  for tau=1:tau_max;
    fprintf(out_file,'evaluating prediction covariance for tau=%d\n',tau);
    %% control printout of diagnostics from MyMatrixPower which just
    %% identically repeat themselves for different tau values:
    if tau==1; diagnostics=1; else; diagnostics=0; end
    for century=1:ncenturies_tau_test;
      %    fprintf(out_file,'century: %d\n',century);
      % Predict the one century that wasn't part of the fit:
      A1=squeeze(A_all_centuries(century,:,:));
      AR1=squeeze(AR1_all_centuries(century,:,:));
      p=(tau/prediction_lead_time_for_fit);
      % matrix exponentiation doesnt work too well, use my own
      % version:
      A1p=MyMatrixPower(A1,p,diagnostics,out_file);
      % eliminate all off diagnonal roundoff terms before raising
      % to power p to prevent soem funny matlab problem:
      AR1p=MyMatrixPower(diag(diag(AR1)),p,diagnostics,out_file);
    
      % check to see that matrix exponiation did not lead to imaginary
      % values:
      t1=max(max(abs(imag(A1 )))); 
      t2=max(max(abs(imag(AR1))));
      if t1>1.e-10 || t2>1.e-12;
        fprintf(out_file,'*** imag(A1,AR1) too large! (%d, %d)\n',t1,t2);
      end
      % get rid of imaginary values due to roundoff:
      A1p=real(A1p); 
      AR1p=real(AR1p); 
      ns=(century-1)*100+1; ne=century*100-tau;
      for n=ns:ne
	% PCs:
        PC_predicted(n+tau,1:I)=PC_n(n,1:I)*A1p;
        residuals(n+tau,1:I)=PC_predicted(n+tau,1:I)-PC_n(n+tau,1:I);
        PC_predicted_AR1(n+tau,1:I)=PC_n(n,1:I)*AR1p;
        residuals_AR1(n+tau,1:I)=PC_predicted_AR1(n+tau,1:I)-PC_n(n+tau,1:I);
	% THC1:
        THC1_predicted(n+tau)=PC_n(n,1:I)*A1p*THC1_PC_regression_coeff;
        THC1_residuals(n+tau)=THC1_predicted(n+tau)-THC1_dimensional(n+tau);
        THC1_predicted_AR1(n+tau)=PC_n(n,1:I)*AR1p*THC1_PC_regression_coeff;
        THC1_residuals_AR1(n+tau)=THC1_predicted_AR1(n+tau)-THC1_dimensional(n+tau);
	% THC2:
        THC2_predicted(n+tau)=PC_n(n,1:I)*A1p*THC2_PC_regression_coeff;
        THC2_residuals(n+tau)=THC2_predicted(n+tau)-THC2_dimensional(n+tau);
        THC2_predicted_AR1(n+tau)=PC_n(n,1:I)*AR1p*THC2_PC_regression_coeff;
        THC2_residuals_AR1(n+tau)=THC2_predicted_AR1(n+tau)-THC2_dimensional(n+tau);
      end
    end
    % calculate covariance matrices of prediction error and of data,
    % as diagnostics of prediction skill for this tau:
    ns=(century-1)*100+1; ne=century*100;
    n=0;
    for nn=ns:ne
      if not(isnan(residuals(nn,1)));
        n=n+1;
	% PCs:
        covar_prediction_error(tau,:,:)=squeeze(covar_prediction_error(tau,:,:)) ...
            +squeeze(residuals(nn,1:I)'*residuals(nn,1:I));
        covar_prediction_error_AR1(tau,:,:)=squeeze(covar_prediction_error_AR1(tau,:,:)) ...
            +squeeze(residuals_AR1(nn,1:I)'*residuals_AR1(nn,1:I));
	% THC1:
        THC1_covar_prediction_error(tau)=squeeze(THC1_covar_prediction_error(tau)) ...
            +squeeze(THC1_residuals(nn)'*THC1_residuals(nn));
        THC1_covar_prediction_error_AR1(tau)= ...
	    squeeze(THC1_covar_prediction_error_AR1(tau)) ...
            +squeeze(THC1_residuals_AR1(nn)'*THC1_residuals_AR1(nn));
	% THC2:
        THC2_covar_prediction_error(tau)=squeeze(THC2_covar_prediction_error(tau)) ...
            +squeeze(THC2_residuals(nn)'*THC2_residuals(nn));
        THC2_covar_prediction_error_AR1(tau)= ...
	    squeeze(THC2_covar_prediction_error_AR1(tau)) ...
            +squeeze(THC2_residuals_AR1(nn)'*THC2_residuals_AR1(nn));
      end
    end
    covar_prediction_error(tau,:,:)=covar_prediction_error(tau,:,:)/(n-1);
    covar_prediction_error_AR1(tau,:,:)=covar_prediction_error_AR1(tau,:,:)/(n-1);
    THC1_covar_prediction_error(tau)=THC1_covar_prediction_error(tau)/(n-1);
    THC1_covar_prediction_error_AR1(tau)=THC1_covar_prediction_error_AR1(tau)/(n-1);
    THC2_covar_prediction_error(tau)=THC2_covar_prediction_error(tau)/(n-1);
    THC2_covar_prediction_error_AR1(tau)=THC2_covar_prediction_error_AR1(tau)/(n-1);
    A_avg=squeeze(mean(A_all_centuries(:,:,:),1)); 
    p=(tau/prediction_lead_time_for_fit);
    A_avg=MyMatrixPower(A_avg,p,diagnostics,out_file);
    if max(max(abs(imag(A_avg))))>1.e-12;
      fprintf(out_file,'*** imag(A_avg) too large!\n');return;
    end
    A_avg=real(A_avg);
    covar_prediction_error_perfect_model(tau,:,:)= ...
        covar_data(:,:) - A_avg'*covar_data*A_avg;
    THC1_covar_prediction_error_perfect_model(tau)=THC1_covar_data - ...
	THC1_PC_regression_coeff'*A_avg'*covar_data*A_avg*THC1_PC_regression_coeff;
    THC2_covar_prediction_error_perfect_model(tau)=THC2_covar_data - ...
	THC2_PC_regression_coeff'*A_avg'*covar_data*A_avg*THC2_PC_regression_coeff;
  end

  %% (3) calculate trace of covariance which then needs to be plotted:
  trace_covar_prediction_error=zeros(tau_max,1);
  trace_covar_prediction_error_AR1=zeros(tau_max,1);
  trace_covar_perfect_model=zeros(tau_max,1);
  for tau=1:tau_max
    trace_covar_prediction_error(tau)=...
        sum(diag(squeeze(covar_prediction_error(tau,:,:))));
    trace_covar_prediction_error_AR1(tau)=...
        sum(diag(squeeze(covar_prediction_error_AR1(tau,:,:))));
    trace_covar_perfect_model(tau)=...
        sum(diag(squeeze(covar_prediction_error_perfect_model(tau,:,:))));
  end
  
  %% (4) plot diagnostics:
  figure; ifigure=1;
  panel_letter=['(a)';'(b)';'(c)';'(d)';'(e)';'(f)';'(g)';'(h)';'(i)'];
  for ii=1:I+2
    jj=mod(ii,9); if jj==0; jj=9; end;
    subplot(3,3,jj);
    if ii==1
      if which_THC_to_use==1
	% THC1 measure of predictability:
	tau_vec=[1:tau_max];
	h=plot(tau_vec,THC1_covar_prediction_error,'-r' ...
	       ,tau_vec,THC1_covar_prediction_error_AR1,'--g' ...
	       ,tau_vec,THC1_covar_prediction_error_perfect_model,':b');
	% legend(gca,'prediction','AR1','perfect model');
	% title('r:pred, g:AR1, b:perfect');
	xlabel('\tau'); ylabel('cov(THC1)');
      elseif which_THC_to_use==2
	% THC2 measure of predictability:
	tau_vec=[1:tau_max];
	h=plot(tau_vec,THC2_covar_prediction_error,'-r' ...
	       ,tau_vec,THC2_covar_prediction_error_AR1,'--g' ...
	       ,tau_vec,THC2_covar_prediction_error_perfect_model,':b');
	% legend(gca,'prediction','AR1','perfect model');
	% title('r:pred, g:AR1, b:perfect');
	xlabel('\tau'); ylabel('cov(THC2)');
      else
	disp('error in which_THC_to_use');
	quit
      end
    elseif ii==2
      % global measure of T,S predictability:
      tau_vec=[1:tau_max];
      h=plot(tau_vec,trace_covar_prediction_error,'-r' ...
             ,tau_vec,trace_covar_prediction_error_AR1,'--g' ...
             ,tau_vec,trace_covar_perfect_model,':b');
      % legend(gca,'prediction','AR1','perfect model');
      % title('r:pred, g:AR1, b:perfect');
      xlabel('\tau'); ylabel('tr(covariance)');
    else
      tau_vec=[1:tau_max];
      h=plot(tau_vec,covar_prediction_error(:,ii-2,ii-2),'-r' ...
             ,tau_vec,covar_prediction_error_AR1(:,ii-2,ii-2),'--g' ...
             ,tau_vec,covar_prediction_error_perfect_model(:,ii-2,ii-2),':b');
      % legend(gca,'prediction','AR1','perfect model');
      % title('r:pred, g:AR1, b:perfect');
      xlabel('\tau'); ylabel(sprintf('covariance(%d,%d)',ii-2,ii-2));
    end
    set(gca,'XTick',[0:20:100]);
    title(panel_letter(jj,:));

    FigureBaseFileName=...
        sprintf('Output/run%2.2d/Figures/tau_test_%2.2d',run_num,ifigure);
    saveas(gcf,[FigureBaseFileName,'.fig']);
    print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

    if floor(ii/9)*9==ii;
      figure; ifigure=ifigure+1;
    end
  end

end % do_tau_test


function A=MyMatrixPower(B,p,diagnostics,out_file);
% calculate A=B^p;

I=length(B(:,1));

% method 1: use eig but special treatment of negative eigenvalues:
% ----------------------------------------------------------------
[V,D] = eig(B);
d=zeros(I,I);
for ii=1:I
  if real(D(ii,ii))>=0 % || abs(imag(D(ii,ii)))>1.e-6;
    d(ii,ii)=D(ii,ii)^p;
  else
    if diagnostics
      format long;
      fprintf(out_file ...
              ,'*** MyMatrixPower, p=%d, resetting eigenvalue=%g+%gi to 0.\n'...
              ,p,real(D(ii,ii)),imag(D(ii,ii)));
    end
    epsilon_regularization=0;
    d(ii,ii)=epsilon_regularization;
  end
end
if cond(V)>1.e10;
  fprintf(out_file,' *** MyMatrixPower: V is ill-conditioned; cond(V)=%d\n' ...
          ,cond(V));
  save('Output/debug.mat','V','B');
  fprintf(out_file,'adding random roundoff noise to B and trying again...\n');
  [V,D] = eig(B+eps*rand(I,I));
  if cond(V)>1.e10
    fprintf(out_file,'sorry, this still does not work, quitting.\n');
    quit;
  else
    fprintf(out_file,'OK, this helped for some reason...\n');
  end
end
A=V*d/V;

% method 2: use eigs:
% --------------------
% BB=zeros(I+5,I+5);
% BB(1:I,1:I)=B;
% opts.disp=0;
% [V,D,flag] = eigs(BB,I+2,'LM',opts);
% if flag~=0;
%   fprintf(out_file,'*** MyMatrixPower: eigenvalues did not converge!!***\n');
%   return
% end
% A=V(1:I,1:I)*D(1:I,1:I).^p/V(1:I,1:I);

% method 3: use eig:
% ------------------
% [V,D]=eig(B); A=V*D.^p/V;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analyze_linearized_nonormal_dynamics ...
    (use_THC_norm_kernel,tau_max,prediction_lead_time_for_fit,run_num,out_file);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(out_file,'analyzing linearized dynamics... \n');

% get the matrix:
load(sprintf('Output/run%2.2d/linearized_model_A_AR1.mat',run_num));
I=length(A(:,1));

% get the norm kernel
if use_THC_norm_kernel
  fprintf(out_file,'using THC norm kernel.\n');
  load(sprintf('Output/run%2.2d/norm_kernel_and_normalized_THC_timeseries.mat',run_num));
  X=X_THC;
else
  fprintf(out_file,'using identity matrix as norm kernel.\n');
  X=eye(I);
end

% calc eigen modes:
[A_eigenvectors,D]=eig(A);
d=diag(D);

fprintf(out_file,'eigenvalues of A (real, imag, abs):\n');
for ii=1:length(diag(D))
  fprintf(out_file,'%2.2d:    %g + %gi; %g\n' ...
	  ,ii,real(D(ii,ii)),imag(D(ii,ii)),abs(D(ii,ii)));
end

fprintf(out_file,'1/eigenvalues of operator whose propagator is A:\n');
fprintf(out_file,'   (that is, A=expm(Operator*prediction_lead_time_for_fit)\n');
% B=exp(A*tau);  Bx=lambda*x ==> x=x0*lambda^n
% dy/dt=Ay=gamma*y ==> y(n*tau)=y0*e^(gamma*n*tau)
% e^[(gamma_r+i*gamma_i)tau]=lambda_r+i*lambda_i  (1)
% ==> abs(lambda_r+i*lambda_i)=e^gamma_r*tau
% ==> gamma_r=log(abs(lambda_r+i*lambda_i))/tau
% real and imag parts of (1):
% lambda_i=e^(gamma_r*tau)*sin(gamma_i*tau)
% lambda_r=e^(gamma_r*tau)*cos(gamma_i*tau)
% ==> lambda_i/lambda_r=tan(gamma_i*tau)
% ==> gamma_i=atan(lambda_i/lambda_r)/tau
tau=prediction_lead_time_for_fit;
for ii=1:length(diag(D))
  lambda_i=imag(d(ii));lambda_r=real(d(ii));
  gamma_i=sign(lambda_i)*atan(abs(lambda_i)/lambda_r)/tau;
  gamma_r=log(abs(lambda_r+i*lambda_i))/tau;
  fprintf(out_file,'%2.2d:  1/gamma_r=%g;  2*pi/gamma_i=%g\n' ...
	  ,ii,1/gamma_r,2*pi/gamma_i);
end
clear tau gamma_r gamma_i lambda_i lambda_r

fprintf(out_file,'making A positive definite:\n');
diagnostics=1;
A=real(MyMatrixPower(A,1,diagnostics,out_file));

% calc eigen modes again:
[A_eigenvectors,D]=eig(A);
d=diag(D);

fprintf(out_file,'eigenvalues of A after making it positive definite:\n');
for ii=1:length(diag(D))
  fprintf(out_file,'%2.2d:    %g + %gi; %g\n' ...
	  ,ii,real(D(ii,ii)),imag(D(ii,ii)),abs(D(ii,ii)));
end

fprintf(out_file,'eigenvectors of A after making it positive definite:\n');
for ii=1:I
  fprintf(out_file,' eigenvector #%g of A:\n',ii);
  fprintf(out_file,'     %g %g %g %g %g %g \n',A_eigenvectors(:,ii));
  fprintf(out_file,'\n');
end

if 0
% first, how non normal is the linearized matrix A? 
angles=zeros(I,I);
for ii=1:I
  for jj=1:I
    % calculate all angles between all eigenvectors of A:
    angles(ii,jj)=acos(A_eigenvectors(:,ii)'*A_eigenvectors(:,jj))*180/pi;
  end
end

fprintf(out_file,' angles between all eigenvectors of A:\n');
fprintf(out_file,'      ',ii); fprintf(out_file,'%3g ',[1:I]); fprintf(out_file,'\n');
for ii=1:I
  fprintf(out_file,'%2d:   ',ii); fprintf(out_file,'%3g ',floor(angles(ii,:))); fprintf(out_file,'\n');
end

% sort by distance from 90 degrees:
n=0;
for ii=1:I
  for jj=1:I
    if ii~=jj
      n=n+1;
      angles_vec(n)=angles(ii,jj);
      ni(n)=ii; nj(n)=jj;
    end
  end
end

dist_from_90deg=abs(angles_vec-90);
[Y,Indx]=sort(dist_from_90deg);
sorted_angles=fliplr(angles_vec(Indx));
fprintf(out_file,' angle     decay rates 1&2:\n');
for ii=1:min(20,I)
  fprintf(out_file,'%g     %g %g\n',sorted_angles(ii),d(ni(ii)),d(nj(ii)));
end
end


% find optimal initial conditions and maximum amplification, as
% function of lead time:
fprintf(out_file,' tau,   largest amplification\n');
optimal_ic=zeros(tau_max,I);
for tau=1:tau_max
  p=(tau/prediction_lead_time_for_fit); 
  if tau==1; diagnostics=1; else; diagnostics=0; end
  Ap=MyMatrixPower(A,p,diagnostics,out_file);
  if use_THC_norm_kernel
    [V1,D1]=eig(Ap*X*(Ap'),X); D1=diag(D1);
  else
    % Matlab seems to react strangely to a generalized eigenproblem
    % with the second matrix being the identity matrix, so I eliminate
    % the second one when it is the identity and is not really needed:
    [V1,D1]=eig(Ap*(Ap')); D1=diag(D1);
  end
  % save('Output/debug.mat','A','Ap','X','V1','D1'); disp('hit C-C now'); pause
  fprintf(out_file,'    %g   %g\n',tau,max(D1));
  [max_amplif(tau),jmax]=max(D1);
  optimal_ic(tau,:)=V1(:,jmax);
  max_anomaly(tau,:)=optimal_ic(tau,:)*Ap;
  fprintf(out_file,'     all eigenvals of Ap''*Ap:\n');
  fprintf(out_file,'     %g %g %g %g %g %g \n',D1);
  fprintf(out_file,'\n');
end

% find tau of maximum amplification:
[global_max_amplif,tau_at_max_amplif]=max(max_amplif);

% print diagnostics on optimal initial conditions, and on solution at
% maximum amplification:
for tau=1:tau_max
  if tau==tau_at_max_amplif; fprintf(out_file,'==> '); end
  fprintf(out_file,'tau=%d; max_amplif(tau)=%g:\n' ...
          ,tau,max_amplif(tau));
  fprintf(out_file,'     norm(i.c.)^2=%g, norm(solution at tau)^2=%g\n' ...
          ,optimal_ic(tau,:)*X*optimal_ic(tau,:)' ...
          ,max_anomaly(tau,:)*X*max_anomaly(tau,:)');
  fprintf(out_file,'     optimal I.C.:\n');
  fprintf(out_file,'     %g %g %g %g %g %g \n',optimal_ic(tau,:));
  fprintf(out_file,'\n');
  fprintf(out_file,'     maximum amplified solution at t=tau:\n');
  fprintf(out_file,'     %g %g %g %g %g %g \n',max_anomaly(tau,:));
  fprintf(out_file,'\n');
end

% calculate and print projection of optimal initial conditions at time
% of maximum amplification on the original eigenmodes of linearized
% model propagator A:
for ii=1:I
  projection(ii)=optimal_ic(tau_at_max_amplif,:)*A_eigenvectors(:,ii);
end
fprintf(out_file,'projection of optimal i.c. for tau=%g on A eivenvectors:\n' ...
        ,tau_at_max_amplif);
fprintf(out_file,'     %g %g %g %g %g %g \n',projection);
fprintf(out_file,'\n');


% save A-eigenvalues for a given value of tau=1yr, so that different
% runs can be compared, and maximum amplification.  Both are later
% plotted by a different program:
p=(1/prediction_lead_time_for_fit); 
diagnostics=0;
Ap=MyMatrixPower(A,p,diagnostics,out_file);
d=eig(Ap);

if use_THC_norm_kernel
  save(sprintf('Output/run%2.2d/max_amplif_X_THC.mat',run_num) ...
     ,'d','A_eigenvectors','max_amplif','tau_at_max_amplif','optimal_ic','max_anomaly');
else
  save(sprintf('Output/run%2.2d/max_amplif.mat',run_num) ...
     ,'d','A_eigenvectors','max_amplif','tau_at_max_amplif','optimal_ic','max_anomaly');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_IC_and_max_anomaly_in_PC_space(use_THC_norm_kernel,out_file,run_num)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot initial conditions in PC space

fprintf(out_file,'plot_IC_and_max_anomaly_in_PC_space, use_THC_norm_kernel=%d\n' ...
	,use_THC_norm_kernel);

if use_THC_norm_kernel
  load(sprintf('Output/run%2.2d/max_amplif_X_THC.mat',run_num));
%     ,'d','max_amplif','tau_at_max_amplif','optimal_ic','max_anomaly'
  title_text='(b)';
  ext='_X_THC';
else
  load(sprintf('Output/run%2.2d/max_amplif.mat',run_num));
%     ,'d','max_amplif','tau_at_max_amplif','optimal_ic','max_anomaly'
  title_text='(a)';
  ext='_energy';
end

figure
axes('position',[0.1 0.3 0.5 0.3]);  % [left, bottom, width, height]
plot(real(optimal_ic(tau_at_max_amplif,:)),'rx');
hold on
plot(real(max_anomaly(tau_at_max_amplif,:)),'bo');
h(1)=xlabel('PC #');
h(2)=ylabel('amplitude');
h(3)=gca;
h(4)=title(title_text);
h(5)=legend('optimal IC','max anomaly','Location','SouthEast');
set(h(1:4),'fontname','helvetica','fontsize',14,'fontweight','bold');
set(h(5),'fontname','helvetica','fontsize',12,'fontweight','bold');
FigureBaseFileName= ...
    sprintf('Output/run%2.2d/Figures/optimal_IC_in_PC_space%s',run_num,ext);
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

fprintf(out_file,'done\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_optimal_ic(use_THC_norm_kernel,run_num,out_file)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load(sprintf('Output/run%2.2d/linearized_model_A_AR1.mat',run_num));
load(sprintf('Output/run%2.2d/EOFs_dimensional.mat',run_num));
load(sprintf('Output/run%2.2d/T_avg_and_std.mat',run_num));
load(sprintf('Output/run%2.2d/masks.mat',run_num));
if use_THC_norm_kernel
  load(sprintf('Output/run%2.2d/max_amplif_X_THC.mat',run_num));
  % The sign of the eigenvectors is arbitrary, and for some reason
  % it comes out opposite when using X_THC, so multiply by -1
  % before plotting:
  optimal_ic=-optimal_ic; max_anomaly=-max_anomaly;
  fprintf(out_file,['plot_optimal_ic: plotting optimal initial conditions and max' ...
		    ' amplified T,S state for THC kernel at year %g\n'],tau_at_max_amplif);
else
  load(sprintf('Output/run%2.2d/max_amplif.mat',run_num));
  fprintf(out_file,['plot_optimal_ic: plotting optimal initial conditions and max' ...
		    ' amplified T,S state for energy kernel at year %g\n'],tau_at_max_amplif);
end
I=length(A(:,1));

% extract optimal initial conditions for T,S from optimal ic for
% principle components:
T_optimal_ic=zeros(nz,ny,nx);
S_optimal_ic=zeros(nz,ny,nx);
% in order to calculate density anomaly, need to add avg T,S and
% substruct avg density:
P_avg=T_avg*0.0; rho_avg=sw_dens(S_avg,T_avg,P_avg);
for e=1:I
  T_optimal_ic=T_optimal_ic ...
      +optimal_ic(tau_at_max_amplif,e) ...
      *squeeze(EOFs_T(e,:,:,:));
  S_optimal_ic=S_optimal_ic ...
      +optimal_ic(tau_at_max_amplif,e) ...
      *squeeze(EOFs_S(e,:,:,:));
  % calc potential density, note that the model temperature is
  % probably potential temperature, which is what requires for
  % sw_dens in order to calculate potential density (see sw_pden):
  PR=0.0*S_optimal_ic; % reference presure is zero
  density_optimal_ic=sw_dens(S_avg+S_optimal_ic,T_avg+T_optimal_ic,PR)-rho_avg;
end
% apply mask:
T_optimal_ic=mask3d.*T_optimal_ic;
S_optimal_ic=mask3d.*S_optimal_ic;
density_optimal_ic=mask3d.*density_optimal_ic;

% Calc T,S at time of maximum amplification:
T_max_anomaly=zeros(nz,ny,nx);
S_max_anomaly=zeros(nz,ny,nx);
for e=1:I
  T_max_anomaly=T_max_anomaly ...
      +max_anomaly(tau_at_max_amplif,e) ...
      *squeeze(EOFs_T(e,:,:,:));
  S_max_anomaly=S_max_anomaly ...
      +max_anomaly(tau_at_max_amplif,e) ...
      *squeeze(EOFs_S(e,:,:,:));
  % calc potential density, note that the model temperature is
  % probably potential temperature, which is what requires for
  % sw_dens in order to calculate potential density (see sw_pden):
  PR=0.0*S_max_anomaly; % reference presure is zero
  density_max_anomaly=sw_dens(S_avg+S_max_anomaly,T_avg+T_max_anomaly,PR)-rho_avg;
end
% apply mask:
T_max_anomaly=mask3d.*T_max_anomaly;
S_max_anomaly=mask3d.*S_max_anomaly;
density_max_anomaly=mask3d.*density_max_anomaly;

% Loop twice, plotting first the optimal initial conditions and
% then the maximum aplified solution for the T,S anomalies:
for iround=1:2
  figure
  subplot(3,3,1);
  % T: apply mask and average in x:
  if iround==1;
    contour_data=nanmean(mask3d.*squeeze(T_optimal_ic(:,:,:)),3);
    title_string=sprintf('(a) ic, T');
  elseif iround==2;
    contour_data=nanmean(mask3d.*squeeze(T_max_anomaly(:,:,:)),3);
    title_string=sprintf('(a) amplified, T');
  end
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  title(title_string);
  colorbar;

  
  subplot(3,3,2);
  % S: apply mask and average in x:
  if iround==1;
    contour_data=nanmean(mask3d.*squeeze(S_optimal_ic(:,:,:)),3);
    title_string=sprintf('(b) ic, S');
  elseif iround==2;
    contour_data=nanmean(mask3d.*squeeze(S_max_anomaly(:,:,:)),3);
    title_string=sprintf('(b) amplified, S');
  end
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  title(title_string);
  colorbar;


  subplot(3,3,3);
  % rho: apply mask and average in x:
  if iround==1;
    contour_data=nanmean(mask3d.*squeeze(density_optimal_ic(:,:,:)),3);
    title_string=sprintf('(c) ic, \\rho');
  elseif iround==2;
    contour_data=nanmean(mask3d.*squeeze(density_max_anomaly(:,:,:)),3);
    title_string=sprintf('(c) amplified, \\rho');
  end
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  title(title_string);
  colorbar;


  k=2; % vertical level at which horizontal plots are made:
  subplot(3,3,4);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % T: apply mask and subsample in z:
  if iround==1;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(T_optimal_ic(k,:,:));
    zz=round(z(k)); title_string=sprintf('(d) ic, T; z=%gm',zz);
  elseif iround==2;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(T_max_anomaly(k,:,:));
    zz=round(z(k)); title_string=sprintf('(d) amplified, T; z=%gm',zz);
  end
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  subplot(3,3,5);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % S: apply mask and subsample in z:
  if iround==1;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(S_optimal_ic(k,:,:));
    zz=round(z(k)); title_string=sprintf('(e) ic, S; z=%gm',zz);
  elseif iround==2;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(S_max_anomaly(k,:,:));
    zz=round(z(k)); title_string=sprintf('(e) amplified, S; z=%gm',zz);
  end
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  subplot(3,3,6);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % rho: apply mask and subsample in z:
  if iround==1;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(density_optimal_ic(k,:,:));
    zz=round(z(k)); title_string=sprintf('(f) ic, \\rho; z=%gm',zz);
  elseif iround==2;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(density_max_anomaly(k,:,:));
    zz=round(z(k)); title_string=sprintf('(f) amplified, \\rho; z=%gm',zz);
  end
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  k=10; % vertical level at which horizontal plots are made:
  subplot(3,3,7);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % T: apply mask and subsample in z:
  if iround==1;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(T_optimal_ic(k,:,:));
    zz=round(z(k)); title_string=sprintf('(g) ic, T; z=%gm',zz);
  elseif iround==2;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(T_max_anomaly(k,:,:));
    zz=round(z(k)); title_string=sprintf('(g) amplified, T; z=%gm',zz);
  end
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  subplot(3,3,8);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % S: apply mask and subsample in z:
  if iround==1;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(S_optimal_ic(k,:,:));
    zz=round(z(k)); title_string=sprintf('(h) ic, S; z=%gm',zz);
  elseif iround==2;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(S_max_anomaly(k,:,:));
    zz=round(z(k)); title_string=sprintf('(h) amplified, S; z=%gm',zz);
  end
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;

  subplot(3,3,9);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % rho: apply mask and subsample in z:
  if iround==1;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(density_optimal_ic(k,:,:));
    zz=round(z(k)); title_string=sprintf('(i) ic, \\rho; z=%gm',zz);
  elseif iround==2;
    contour_data=squeeze(mask3d(k,:,:)).*squeeze(density_max_anomaly(k,:,:));
    zz=round(z(k)); title_string=sprintf('(i) amplified, \\rho; z=%gm',zz);
  end
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;

  if use_THC_norm_kernel
    file_name_extention='_X_THC';
  else
    file_name_extention='';
  end

  if iround==1;
    FigureBaseFileName=sprintf('Output/run%2.2d/Figures/TS_optimal_ic%s' ...
                               ,run_num,file_name_extention);
  elseif iround==2;
    FigureBaseFileName=sprintf('Output/run%2.2d/Figures/TS_max_amplif%s' ...
                               ,run_num,file_name_extention);
  end
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
  % system(['./eps2homepage ' FigureBaseFileName]);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_A_eigenvectors_3dTS(run_num,out_file)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% modified from plot_optimal_ic

load(sprintf('Output/run%2.2d/linearized_model_A_AR1.mat',run_num));
load(sprintf('Output/run%2.2d/EOFs_dimensional.mat',run_num));
load(sprintf('Output/run%2.2d/T_avg_and_std.mat',run_num));
load(sprintf('Output/run%2.2d/masks.mat',run_num));
load(sprintf('Output/run%2.2d/max_amplif.mat',run_num));
fprintf(out_file,['plot_A_eigenvectors_3dTS: contouring eigenvectors' ...
		    ' of propagator in T,S space.\n']);
I=length(A(:,1));

for eigenvec_num=1:I
  % calc eigenvalues in T,S space from eigenvectors in PC space:
  T_eigenvec=zeros(nz,ny,nx);
  S_eigenvec=zeros(nz,ny,nx);
  % in order to calculate density anomaly, need to add avg T,S and
  % substruct avg density:
  P_avg=T_avg*0.0; rho_avg=sw_dens(S_avg,T_avg,P_avg);
  for e=1:I
    % The following assumes that the eigenvectors are stored as the
    % columns of A_eigenvectors:
    T_eigenvec=T_eigenvec ...
      +A_eigenvectors(e,eigenvec_num) ...
      *squeeze(EOFs_T(e,:,:,:));
    S_eigenvec=S_eigenvec ...
	+A_eigenvectors(e,eigenvec_num) ...
	*squeeze(EOFs_S(e,:,:,:));
    % calc potential density, note that the model temperature is
    % probably potential temperature, which is what requires for
    % sw_dens in order to calculate potential density (see sw_pden):
    PR=0.0*S_eigenvec; % reference presure is zero
    density_eigenvec=sw_dens(S_avg+S_eigenvec,T_avg+T_eigenvec,PR)-rho_avg;
  end
  % apply mask:
  T_eigenvec=mask3d.*T_eigenvec;
  S_eigenvec=mask3d.*S_eigenvec;
  density_eigenvec=mask3d.*density_eigenvec;

  figure
  subplot(3,3,1);
  % T: apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(T_eigenvec(:,:,:)),3);
  title_string=sprintf('(a) vec %d, T',eigenvec_num);
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  title(title_string);
  colorbar;

  
  subplot(3,3,2);
  % S: apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(S_eigenvec(:,:,:)),3);
  title_string=sprintf('(b) vec %d, S',eigenvec_num);
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  title(title_string);
  colorbar;


  subplot(3,3,3);
  % rho: apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(density_eigenvec(:,:,:)),3);
  title_string=sprintf('(c) vec %d, \\rho',eigenvec_num);
  % contour:
  [c,h]=contourf(y,z/1000,contour_data);
  clear contour_data
  set(gca,'YDir','reverse');
  ylabel('depth (km)');
  set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  title(title_string);
  colorbar;


  k=2; % vertical level at which horizontal plots are made:
  subplot(3,3,4);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % T: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(T_eigenvec(k,:,:));
  zz=round(z(k)); title_string=sprintf('(d) vec %d, T; z=%gm',eigenvec_num,zz);
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  subplot(3,3,5);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % S: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(S_eigenvec(k,:,:));
  zz=round(z(k)); title_string=sprintf('(e) vec %d, S; z=%gm',eigenvec_num,zz);
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  subplot(3,3,6);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % rho: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(density_eigenvec(k,:,:));
  zz=round(z(k)); title_string=sprintf('(f) vec %d, \\rho; z=%gm',eigenvec_num,zz);
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  k=10; % vertical level at which horizontal plots are made:
  subplot(3,3,7);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % T: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(T_eigenvec(k,:,:));
  zz=round(z(k)); title_string=sprintf('(g) vec %d, T; z=%gm',eigenvec_num,zz);
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;


  subplot(3,3,8);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % S: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(S_eigenvec(k,:,:));
  zz=round(z(k)); title_string=sprintf('(h) vec %d, S; z=%gm',eigenvec_num,zz);
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;

  subplot(3,3,9);
  %m_proj('lambert','lat',[y(1) y(ny)],'long',[x(1) x(nx)-15]);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',8);
  hold on
  % rho: apply mask and subsample in z:
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(density_eigenvec(k,:,:));
  zz=round(z(k)); title_string=sprintf('(i) vec %d, \\rho; z=%gm',eigenvec_num,zz);
  % contour:
  m_contourf(x,y,contour_data);
  clear contour_data
  title(title_string);
  colorbar;

  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/TS_eigenvec_%2.2d' ...
			     ,run_num,eigenvec_num);
  saveas(gcf,[FigureBaseFileName,'.fig']);
  print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function PC=smooth_PCs(PC,t_run_avg,out_file);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function is called every time PCs need to be used, after they are
% loaded from PCs.mat file, in order to smooth them using a weighted running
% average.

fprintf(out_file,'smoothing PCs...');
N=length(PC(:,1));
I=length(PC(1,:));

for t=1:N
  tmp=zeros(I,1);
  tot_wgt=0;
  for tt=max(t-2*t_run_avg,1):min(N,t+2*t_run_avg);
    wgt=exp(-(t-tt)^2/t_run_avg^2);
    tot_wgt=tot_wgt+wgt;
    tmp=tmp+wgt*squeeze(PC(tt,:))';
  end
  PC(t,:)=tmp/tot_wgt;
end
fprintf(out_file,' done.\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_linear_model_oscillation(n_PCs_to_fit,prediction_lead_time_for_fit,tau_max,use_THC_norm_kernel,run_num,out_file);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot evolution of the linearized model starting from the optimal
% initial conditions, during an oscillation of the THC.  Plot in
% both T,S space and in reduced EOF space.

fprintf(out_file,'Entered plot_linear_model_oscillation..., use_THC_norm_kernel=%d\n' ...
	,use_THC_norm_kernel);

% get propagator, EOFs and optimal initial conditions:
load(sprintf('Output/run%2.2d/linearized_model_A_AR1.mat',run_num));
load(sprintf('Output/run%2.2d/EOFs_dimensional.mat',run_num));
load(sprintf('Output/run%2.2d/T_avg_and_std.mat',run_num));
load(sprintf('Output/run%2.2d/masks.mat',run_num));
if use_THC_norm_kernel
  load(sprintf('Output/run%2.2d/max_amplif_X_THC.mat',run_num));
else
  load(sprintf('Output/run%2.2d/max_amplif.mat',run_num));
end
load(sprintf('Output/run%2.2d/norm_kernel_and_normalized_THC_timeseries.mat',run_num));

% run linear model and save history in reduced EOF space:
dtau=1;itau=0;
for tau=0:dtau:100
  itau=itau+1;
  tplot(itau)=tau;
  p=(tau/prediction_lead_time_for_fit); 
  if tau==0; diagnostics=1; else; diagnostics=0; end
  Ap=MyMatrixPower(A,p,diagnostics,out_file);
  anomaly_PC(itau,:)=optimal_ic(tau_at_max_amplif,:)*Ap;
  anomaly_THC2(itau)=anomaly_PC(itau,1:n_PCs_to_fit)*THC2_PC_regression_coeff;
end

if max(abs(imag(anomaly_THC2)))<1.e-10
  anomaly_THC2=real(anomaly_THC2);
end

% plot reduced EOF history:
figure;
nPCaxis=1:n_PCs_to_fit;
subplot(1,3,[1 2])
[c,h]=contourf(nPCaxis,tplot,real(anomaly_PC));
for i=1:length(h); set(h(i),'LineStyle','none');end
h1=xlabel('PC #');
h2=ylabel('time (yr)');
if use_THC_norm_kernel
  h3=title('(c)');
else
  h3=title('(a)');
end
h4=colorbar;
set([gca,h1,h2,h3,h4],'fontsize',14);

subplot(1,3,3)
plot(anomaly_THC2*1.e3,tplot)
h1=xlabel('THC');
h2=ylabel('time (yr)');
if use_THC_norm_kernel
  h3=title('(d)');
else
  h3=title('(b)');
end
set([gca,h1,h2,h3],'fontsize',14);

if use_THC_norm_kernel
  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/oscillation_EOF_X_THC',run_num);
else
  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/oscillation_EOF',run_num);
end
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

% convert EOF history to T,S history:
% extract optimal initial conditions for T,S from optimal ic for
% principle components:
itau_end=34;dtau=1;
anomaly_T=zeros(nz,ny,nx,itau_end);
anomaly_S=zeros(nz,ny,nx,itau_end);
anomaly_density=zeros(nz,ny,nx,itau_end);
anomaly_specific_vlume=zeros(nz,ny,nx,itau_end);
% in order to calculate density anomaly, need to add avg T,S and
% substruct avg density:
I=length(A(:,1));
P_avg=T_avg*0.0; rho_avg=sw_dens(S_avg,T_avg,P_avg);
for itau=1:itau_end
  for e=1:I
    anomaly_T(:,:,:,itau)=anomaly_T(:,:,:,itau) ...
	+anomaly_PC(itau,e) ...
	*squeeze(EOFs_T(e,:,:,:));
    anomaly_S(:,:,:,itau)=anomaly_S(:,:,:,itau) ...
	+anomaly_PC(itau,e) ...
	*squeeze(EOFs_S(e,:,:,:));
    % calc potential density, note that the model temperature is
    % probably potential temperature, which is what requires for
    % sw_dens in order to calculate potential density (see sw_pden):
    PR=0.0*anomaly_S(:,:,:,1); % reference presure is zero
    anomaly_specific_volume(:,:,:,itau)=1./sw_dens(S_avg+anomaly_S(:,:,:,itau) ...
				      ,T_avg+anomaly_T(:,:,:,itau),PR)-1./rho_avg;
    anomaly_density(:,:,:,itau)= ...
	sw_dens(S_avg+anomaly_S(:,:,:,itau),T_avg+anomaly_T(:,:,:,itau),PR) ...
	-rho_avg;
  end
  % apply mask:
  anomaly_T(:,:,:,itau)=mask3d.*real(anomaly_T(:,:,:,itau));
  anomaly_S(:,:,:,itau)=mask3d.*real(anomaly_S(:,:,:,itau));
  anomaly_density(:,:,:,itau)=mask3d.*real(anomaly_density(:,:,:,itau));
  anomaly_specific_volume(:,:,:,itau)=mask3d.*real(anomaly_specific_volume(:,:,:,itau));
end

% Calculate history of surface dynamic topography anomaly from the
% anomaly specific volume history:
anomaly_dynamic_topography=zeros(ny,nx,itau_end);
mean_dynamic_topography=zeros(ny,nx);
pressure=zeros(nz,ny,nx);
g=9.8; nref=10;  %% a seemingly reasonable reference level: z(10)=1364m
for itau=1:itau_end
  for k=1:nref
    if itau==1
      %% first, for calculation of mean dynamic topography:
      if k==1
	mean_specific_volume= ...
	    1./sw_dens(S_avg,T_avg,squeeze(PR)) ...
	    -1./sw_dens(35.00,4.0,0.0);
      end

      %% next, calculate pressure:
      if k==1
	pressure(k,:,:)=0;
      else
	pressure(k,:,:)=pressure(k-1,:,:)+g*rho_avg(k,:,:)*(z(k)-z(k-1));
      end
    end
    %% dynamic topography is given by the integral over pressure: 
    %% Delta D=\int dp \delta/ 10;  
    %% where delta=1/\rho-1/\rho_0
    if k>1; 
      dp=pressure(k,:,:)-pressure(k-1,:,:); 
    %  size(anomaly_dynamic_topography(:,:,itau))
    %  size(dp), size(anomaly_specific_volume(k,:,:,itau))
      anomaly_dynamic_topography(:,:,itau)= ...
	  anomaly_dynamic_topography(:,:,itau) ...
	  + squeeze(dp.*anomaly_specific_volume(k,:,:,itau)/10);
      if itau==1
	%% calculate the mean dynamic topography:
	mean_dynamic_topography(:,:)= ...
	    mean_dynamic_topography(:,:) ...
	    + squeeze(dp.*mean_specific_volume(k,:,:)/10);
      end
    end
  end
  % apply mask:
  anomaly_dynamic_topography(:,:,itau)= ...
      mask.*squeeze(anomaly_dynamic_topography(:,:,itau));
end

% plot mean dynamic topography:
figure
m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
m_grid('linestyle','none','tickdir','in','xtick',[-100:40:30],'fontsize',10);
hold on
contour_data=mask(:,:).*squeeze(mean_dynamic_topography(:,:));
[c,h]=m_contourf(x,y,contour_data);
for i=1:length(h); set(h(i),'LineStyle','none');end
title('mean Dynamic Topog');
colorbar;
if use_THC_norm_kernel
  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/Dynamic_Topography_mean_X_THC',run_num);
else
  FigureBaseFileName=sprintf('Output/run%2.2d/Figures/Dynamic_Topography_mean',run_num);
end
saveas(gcf,[FigureBaseFileName,'.fig']);
print(gcf,'-depsc',[FigureBaseFileName,'.eps']);

% plot T,S and dynamic topography history:
% ----------------------------------------
ifig=0;ipage=0; rows_per_page=7; panels_per_row=6;
nx_panels=panels_per_row;ny_panels=rows_per_page;
%% choose contour levels for each type of plot (based on max a-f
%% and min a-f in the output.txt file)
if use_THC_norm_kernel
  ContInt(1,:)=linspace(-1.6,1.12,10); ContInt(2,:)=linspace(-0.38,0.26,10); 
  ContInt(3,:)=linspace(-0.72,0.52,10); ContInt(4,:)=linspace(-0.13,0.12,10); 
  ContInt(5,:)=linspace(-0.065,0.053,10); ContInt(6,:)=linspace(-0.05,0.089,10); 
else
  ContInt(1,:)=linspace(-0.26,0.73,10); ContInt(2,:)=linspace(-0.063,0.17,10); 
  ContInt(3,:)=linspace(-0.23,0.3,10); ContInt(4,:)=linspace(-0.05,0.06,10); 
  ContInt(5,:)=linspace(-0.021,0.038,10); ContInt(6,:)=linspace(-0.031,0.023,10); 
end
% modify the above such that zero corresponds to the same place in
% the color map for all plots:
lci=length(ContInt(1,:));
for ici=1:6
  if -ContInt(ici,1)>ContInt(ici,lci)
    ContInt(ici,:)=linspace(ContInt(ici,1),-ContInt(ici,1),10);
  else
    ContInt(ici,:)=linspace(-ContInt(ici,lci),ContInt(ici,lci),10);
  end
end

%% prepare to calculate max/min of each type of plot for all times:
maxXXa=-100;maxXXb=-100;maxXXc=-100;maxXXd=-100;maxXXe=-100;maxXXf=-100;
minXXa= 100;minXXb= 100;minXXc= 100;minXXd= 100;minXXe= 100;minXXf= 100;
% choose times to plot state for the paper:
if use_THC_norm_kernel
  itau_values=1:2:13;
else
  itau_values=[1:2:11,28];
end
for itau=itau_values
  ifig=ifig+1;
  if mod(ifig,rows_per_page)==1; 
    figure; 
    set(gcf,'PaperUnits','inches'); 
    set(gcf,'PaperType','usletter'); 
    set(gcf,'PaperPosition',[0.5 0.5 7.5 10]); 
    ipage=ipage+1;
  end

  % horizontal temperature section:
  ifig_row=mod(ifig,rows_per_page); if ifig_row==0; ifig_row=rows_per_page; end
  %% eliminate some of the white space around the sub-plots:
  my_subplot(rows_per_page,panels_per_row,panels_per_row*(ifig_row-1)+1);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[],'ytick',[],'fontsize',10); %% 'xtick',[-100:40:30]
  hold on
  k=2; 
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(anomaly_T(k,:,:,itau));
  cmax=nanmax(nanmax(contour_data));  cmin=nanmin(nanmin(contour_data));
  maxXXa=max(maxXXa,cmax);
  minXXa=min(minXXa,cmin);
  [c,h]=m_contourf(x,y,real(contour_data),ContInt(1,:));
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  axis square;
  %% make sure color scale is the same for all contour plots:
  set(gca,'clim',[ContInt(1,1),ContInt(1,length(ContInt(1,:)))]); 
  if 1
    ContInt(1,:)=get(h,'LevelList'); 
    fprintf(out_file,'itau=%d; c.i.=',itau);fprintf(out_file,'%g; ',ContInt(1,:));fprintf(out_file,'\n');
    fprintf(out_file,'min=%g; max=%g\n',cmin,cmax);
    if cmin<ContInt(1,1) || cmax >ContInt(1,length(ContInt(1,:))); fprintf(out_file,'***\n'); end
  end
  for i=1:length(h); set(h(i),'LineStyle','none');end
  h=title(sprintf('t=%d: T(45m),',tplot(itau)));
  % move title a bit downward:
  set(h,'Units','normalized'); TPos=get(h,'Position');set(h,'Position',[TPos(1) 0.88*TPos(2) TPos(3)]);

  
  % horizontal salinity section:
  ifig_row=mod(ifig,rows_per_page); if ifig_row==0; ifig_row=rows_per_page; end
  %% eliminate some of the white space around the sub-plots:
  my_subplot(rows_per_page,panels_per_row,panels_per_row*(ifig_row-1)+2);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[],'ytick',[],'fontsize',10); %% 'xtick',[-100:40:30]
  hold on
  k=2;
  contour_data=squeeze(mask3d(k,:,:)).*squeeze(anomaly_S(k,:,:,itau));
  cmax=nanmax(nanmax(contour_data));  cmin=nanmin(nanmin(contour_data));
  maxXXb=max(maxXXb,cmax);
  minXXb=min(minXXb,cmin);
  [c,h]=m_contourf(x,y,real(contour_data),ContInt(2,:));
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  axis square;
  %% make sure color scale is the same for all contour plots:
  set(gca,'clim',[ContInt(2,1),ContInt(2,length(ContInt(2,:)))]); 
  if 1
    ContInt(2,:)=get(h,'LevelList');     
    fprintf(out_file,'itau=%d; c.i.=',itau);fprintf(out_file,'%g; ',ContInt(2,:));fprintf(out_file,'\n');
    fprintf(out_file,'min=%g; max=%g\n',cmin,cmax);
    if cmin<ContInt(2,1) || cmax >ContInt(2,length(ContInt(2,:))); fprintf(out_file,'***\n'); end
  end
  for i=1:length(h); set(h(i),'LineStyle','none');end
  h=title('S(45m)');
  % move title a bit downward:
  set(h,'Units','normalized'); TPos=get(h,'Position');set(h,'Position',[TPos(1) 0.88*TPos(2) TPos(3)]);


  % vertical temperature section:
  ifig_row=mod(ifig,rows_per_page); if ifig_row==0; ifig_row=rows_per_page; end
  %% eliminate some of the white space around the sub-plots:
  my_subplot(rows_per_page,panels_per_row,panels_per_row*(ifig_row-1)+3);
  % apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(anomaly_T(:,:,:,itau)),3);
  cmax=nanmax(nanmax(contour_data));  cmin=nanmin(nanmin(contour_data));
  maxXXc=max(maxXXc,cmax);
  minXXc=min(minXXc,cmin);
  [c,h]=contourf(y,z/1000,real(contour_data),ContInt(3,:));
  axis square;
  %% make sure color scale is the same for all contour plots:
  set(gca,'clim',[ContInt(3,1),ContInt(3,length(ContInt(3,:)))]); 
  if 1
    ContInt(3,:)=get(h,'LevelList'); 
    fprintf(out_file,'itau=%d; c.i.=',itau);fprintf(out_file,'%g; ',ContInt(3,:));fprintf(out_file,'\n');
    fprintf(out_file,'min=%g; max=%g\n',cmin,cmax);
    if cmin<ContInt(3,1) || cmax >ContInt(3,length(ContInt(3,:))); fprintf(out_file,'***\n'); end
  end
  for i=1:length(h); set(h(i),'LineStyle','none');end
  clear contour_data
  set(gca,'YDir','reverse');
  set(gca,'XTick',[],'YTick',[]);
  % set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  h=title('T');
  % move title a bit downward:
  set(h,'Units','normalized'); TPos=get(h,'Position');set(h,'Position',[TPos(1) 0.88*TPos(2) TPos(3)]);

  % vertical salinity section:
  ifig_row=mod(ifig,rows_per_page); if ifig_row==0; ifig_row=rows_per_page; end
  %% eliminate some of the white space around the sub-plots:
  my_subplot(rows_per_page,panels_per_row,panels_per_row*(ifig_row-1)+4);
  % apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(anomaly_S(:,:,:,itau)),3);
  cmax=nanmax(nanmax(contour_data));  cmin=nanmin(nanmin(contour_data));
  maxXXd=max(maxXXd,cmax);
  minXXd=min(minXXd,cmin);
  [c,h]=contourf(y,z/1000,real(contour_data),ContInt(4,:));
  axis square;
  %% make sure color scale is the same for all contour plots:
  set(gca,'clim',[ContInt(4,1),ContInt(4,length(ContInt(4,:)))]); 
  if 1
    ContInt(4,:)=get(h,'LevelList');
    fprintf(out_file,'itau=%d; c.i.=',itau);fprintf(out_file,'%g; ',ContInt(4,:));fprintf(out_file,'\n');
    fprintf(out_file,'min=%g; max=%g\n',cmin,cmax);
    if cmin<ContInt(4,1) || cmax >ContInt(4,length(ContInt(4,:))); fprintf(out_file,'***\n'); end
  end
  for i=1:length(h); set(h(i),'LineStyle','none');end
  clear contour_data
  set(gca,'YDir','reverse');
  set(gca,'XTick',[],'YTick',[]);
  % set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  h=title('S');
  % move title a bit downward:
  set(h,'Units','normalized'); TPos=get(h,'Position');set(h,'Position',[TPos(1) 0.88*TPos(2) TPos(3)]);

  % vertical density section:
  ifig_row=mod(ifig,rows_per_page); if ifig_row==0; ifig_row=rows_per_page; end
  %% eliminate some of the white space around the sub-plots:
  my_subplot(rows_per_page,panels_per_row,panels_per_row*(ifig_row-1)+5);
  % apply mask and average in x:
  contour_data=nanmean(mask3d.*squeeze(anomaly_density(:,:,:,itau)),3);
  cmax=nanmax(nanmax(contour_data));  cmin=nanmin(nanmin(contour_data));
  maxXXe=max(maxXXe,cmax);
  minXXe=min(minXXe,cmin);
  [c,h]=contourf(y,z/1000,real(contour_data),ContInt(5,:));
  axis square;
  %% make sure color scale is the same for all contour plots:
  set(gca,'clim',[ContInt(5,1),ContInt(5,length(ContInt(5,:)))]); 
  if 1
    ContInt(5,:)=get(h,'LevelList');
    fprintf(out_file,'itau=%d; c.i.=',itau);fprintf(out_file,'%g; ',ContInt(5,:));fprintf(out_file,'\n');
    fprintf(out_file,'min=%g; max=%g\n',cmin,cmax);
    if cmin<ContInt(5,1) || cmax >ContInt(5,length(ContInt(5,:))); fprintf(out_file,'***\n'); end
  end
  for i=1:length(h); set(h(i),'LineStyle','none');end
  clear contour_data
  set(gca,'YDir','reverse');
  set(gca,'XTick',[],'YTick',[]);
  % set(gca,'XTick',[20 40 60 80],'XTickLabel',{'20';'40';'60';'80N'});
  h=title('\rho');
  % move title a bit downward:
  set(h,'Units','normalized'); TPos=get(h,'Position');set(h,'Position',[TPos(1) 0.88*TPos(2) TPos(3)]);

  % dynamic topography:
  ifig_row=mod(ifig,rows_per_page); if ifig_row==0; ifig_row=rows_per_page; end
  %% eliminate some of the white space around the sub-plots:
  Pos=my_subplot(rows_per_page,panels_per_row,panels_per_row*(ifig_row-1)+6);
  m_proj('miller','lat',[y(1) y(ny)-5],'long',[x(1) x(nx)-10]);
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  m_grid('linestyle','none','tickdir','in','xtick',[],'ytick',[],'fontsize',10); %% 'xtick',[-100:40:30]
  hold on
  k=2;
  contour_data=mask(:,:).*squeeze(anomaly_dynamic_topography(:,:,itau));
  cmax=nanmax(nanmax(contour_data));  cmin=nanmin(nanmin(contour_data));
  maxXXf=max(maxXXf,cmax);
  minXXf=min(minXXf,cmin);
  [c,h]=m_contourf(x,y,real(contour_data),ContInt(6,:));
  m_coast('patch',[0.7 0.7 0.7],'edgecolor','none');
  axis square;
  %% make sure color scale is the same for all contour plots:
  set(gca,'clim',[ContInt(6,1),ContInt(6,length(ContInt(6,:)))]); 
  if 1
    ContInt(6,:)=get(h,'LevelList'); 
    fprintf(out_file,'itau=%d; c.i.=',itau);fprintf(out_file,'%g; ',ContInt(6,:));fprintf(out_file,'\n');
    fprintf(out_file,'min=%g; max=%g\n',cmin,cmax);
    if cmin<ContInt(6,1) || cmax >ContInt(6,length(ContInt(6,:))); fprintf(out_file,'***\n'); end
  end
  for i=1:length(h); set(h(i),'LineStyle','none');end
  h=title('Dyn Top'); 
  % move title a bit downward:
  set(h,'Units','normalized'); TPos=get(h,'Position');set(h,'Position',[TPos(1) 0.88*TPos(2) TPos(3)]);
  if ifig_row==1
    h=colorbar;
    set(h,'Location','SouthOutside','Position', [0.25 0.06 0.5 0.03] ...
	  ,'XTick',[0]); %,'XTickLabel',[]);
 end
  
  fprintf(out_file,'---\n');

  if mod(ifig,rows_per_page)==0 || itau==itau_end
    if use_THC_norm_kernel
      FigureBaseFileName=sprintf('Output/run%2.2d/Figures/oscillation_T_S_DT_X_THC_%2.2d',run_num,ipage);
    else
      FigureBaseFileName=sprintf('Output/run%2.2d/Figures/oscillation_T_S_DT_%2.2d',run_num,ipage);
    end
    saveas(gcf,[FigureBaseFileName,'.fig']);
    print(gcf,'-depsc',[FigureBaseFileName,'.eps']);
  end
end

fprintf(out_file,'max a-f=%g, %g, %g, %g, %g, %g\n',maxXXa,maxXXb,maxXXc,maxXXd,maxXXe,maxXXf);
fprintf(out_file,'min a-f=%g, %g, %g, %g, %g, %g\n',minXXa,minXXb,minXXc,minXXd,minXXe,minXXf);

close all

fprintf(out_file,'Exiting plot_linear_model_oscillation.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Pos=my_subplot(nrows,ncols,ifig)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replace Matlab's subplot, to allow a better control of spacing
% between panels.  Eli, June 12, 2007.
%fprintf(1,'ir,jc,nx_panels,ny_panels=%d, %d, %d, %d\n',ir,jc,nx_panels,ny_panels)
% left and right margins of page:
x0=0.05; 
% top margin of page:
y_top_margin=0.05;
% bottom margin of page:
y_bot_margin=0.1;
% left and right margins of each panel (relative to panel width):
dx=0.05;
% top and bottom margins of each panel (relative to panel height):
dy=0.1;
% width and height of entire plot on page:
W=1-x0*2; H=1-(y_top_margin+y_bot_margin);
% separation in x and y between panels:
xsep=W/ncols; ysep=H/nrows;
% width and height of each pannel:
width=xsep*(1-2*dx); height=ysep*(1-2*dy);
% indices of row and column of the panel being drawn:
ir=floor((ifig-1)/ncols)+1; jc=mod(ifig,ncols); if jc==0; jc=ncols; end
% position:
left=x0+xsep*(jc-1); bottom=(1-y_top_margin)-ysep*ir;
Pos=[left bottom width height];
axes('Position',Pos);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [do_read_raw_netcdf_data, do_read_TS_data, ncenturies ...
              ,do_avg_instead_of_subsample ...
              ,read_mask, print_raw_mask, do_contour_TS, do_calc_EOFs ...
              ,do_apply_grid_weights, do_normalize_by_std ...
              ,do_calc_PCs, do_calc_PCs_from_smoothed_data, do_smooth_PCs ...
              ,do_dimensionalize_EOFs, do_use_normalized_PCs ...
              ,do_analyze_THC_index, do_filter_in_analyze_THC_index, do_plot_PCs ...
              ,which_THC_to_use, do_calc_linear_fit, plot_in_fit_linear_model ...
              ,plot_obs_and_predicted_PCs, n_PCs_to_fit ...
              ,smooth_b4_plotting_predicted_PCs, prediction_start_interval ...
              ,prediction_lead_time_for_fit, tau_max ...
              ,do_eval_fit_quality, do_tau_test, do_analyze_linarized_dynamics ...
              ,do_plot_optimal_ic, do_plot_linear_model_oscillation ...
              ,do_contour_EOFs, data_dir, netcdf_path ...
              ,subsample_x, subsample_y, subsample_z, subsample_t, t_run_avg ...
              ,do_smooth_data_in_time,nsmooth_passes ...
              ,nx_b, nx_e, ny_b, ny_e, nz_b, nz_e, nt_b, nt_e, n_EOFs]= ...
              set_parameters(out_file,run_num);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set parameters:
fprintf(out_file,'setting parameters...\n');

% which computer?
computer=getenv('HOSTNAME');
if strcmp(computer,''); computer='cygwin'; end;

% ----------------
% main parameters:
% ----------------------
if run_num==0
  ncenturies=20;
  do_read_TS_data=0;
  do_read_raw_netcdf_data=0;
  do_avg_instead_of_subsample=1;
  do_apply_grid_weights=1;
  do_normalize_by_std=1;
  read_mask=1;
  print_raw_mask=0;
  do_contour_TS=0;
  n_EOFs=30;
  do_calc_EOFs=0;
  do_contour_EOFs=0;
  do_smooth_data_in_time=1;
  t_run_avg=15;
  nsmooth_passes=3;
  do_calc_PCs=0;
  do_calc_PCs_from_smoothed_data=0;
  do_smooth_PCs=1;
  do_plot_PCs=0;
  do_dimensionalize_EOFs=1;
  do_analyze_THC_index=0;
  which_THC_to_use=1;
  do_use_normalized_PCs=0;
  do_filter_in_analyze_THC_index=0;
  do_calc_linear_fit=1;
  plot_in_fit_linear_model=0;
  do_eval_fit_quality=1;
  do_tau_test=1;
  plot_obs_and_predicted_PCs=1;
  smooth_b4_plotting_predicted_PCs=0;
  prediction_start_interval=15;
  n_PCs_to_fit=12;
  prediction_lead_time_for_fit=40;
  tau_max=120;
  do_analyze_linarized_dynamics=1;
  do_plot_optimal_ic=1;
  do_plot_linear_model_oscillation=1;
else
  fprintf(out_file,'reading parameters from file Input/run%2.2d_input.m\n',run_num);
  run(sprintf('Input/run%2.2d_input.m',run_num));
end
% ----------------------

% computer-dependent settings:
if strcmp(computer,'beach');
  data_dir='/data/eli/GFDL_CM2/';
  netcdf_path='';
elseif strcmp(computer,'swell');
  if do_read_raw_netcdf_data
    data_dir='/misc/data_eli/gebbie/CM2/';
  else
    data_dir='/misc/data_eli/eli/CM2/';
  end
  netcdf_path='';
elseif strcmp(computer,'cygwin');
  data_dir='/cygwin/home/eli/GFDL_CM2_data/';
  netcdf_path='/cygwin'; % on laptop:
end

% Setup path for matlab routines that deal with netcdf files:
% -----------------------------------------------------------
if strcmp(computer,'cygwin') || strcmp(computer,'beach ');
  addpath([netcdf_path '/usr/local/mexnc'],'-end');
  addpath([netcdf_path '/home/eli/matlab/netcdf_toolbox/netcdf'],'-end');
  addpath([netcdf_path '/home/eli/matlab/netcdf_toolbox/netcdf/nctype'],'-end');
  addpath([netcdf_path '/home/eli/matlab/netcdf_toolbox/netcdf/ncutility'],'-end');
end

% which('mexnc','-all') % when I uncomment this, the mexnc.m should
                        % be "shadowed" 

% subsampling in all directions:
subsample_x=3; subsample_y=3; subsample_z=4; subsample_t=1;

% index range covering the Atlantic ocean:
nx_b=180; nx_e=320; ny_b=110; ny_e=200; nz_b=1; nz_e=50; nt_b=501; nt_e=2000;

fprintf(out_file,' done.\n');
